! --------------------------------------------------------------------------------------------------------------------
! This file is part of the package EMar, developed to produce the results in the paper "The marriage market, labor 
!    supply and education choice," by P.A. Chiapppori, M. Costa-Dias and C. Meghir. 
! Copyright (c) 2017 Monica Costa Dias
!
! Routines to import and export data
! --------------------------------------------------------------------------------------------------------------------


module IOroutines

use paths
use precision,    only : wp, tol
use constants,    only : mina, maxa, nh, neta, ns, ntheta, ng
use parameters,   only : param_t, paramS, nParamMax
use momData,      only : mom_t, momWL_t, momSpH_t, PSmSf, PSmSf_se, PSmSf_sim
use SimStructure, only : nobsLC, nrepsWL, simLC
use stataMod

contains 


    ! ----------------------------------------------------------------------------------------------------------------
    ! Parameters
    !

    !
    ! Load parameters
    ! 
    subroutine paramLoadStarting(par,paramFile)
        implicit none
        character(*),  intent(in)  :: paramFile
        type(param_t), intent(out) :: par
        integer                    :: unit, iParam, ios
        character(160)             :: buffer

        ! Get unit and open parameter file
        unit = 749
        open (unit, file=paramFile, action='read')
        
        ! Read parameters
        iParam = 0
        do

            read(unit, '(A160)', iostat=ios) buffer
            if (ios < 0) exit

            if (.not.((buffer == "") .or. (buffer(1:1) == "!"))) then
                iParam = iParam + 1
                read(buffer, '(T1, A15, T17, F15.8, T35, F15.8, T53, F15.8, T71, L1)')      &
                    par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
            end if

        end do
        par%n           = iParam
        par%nToEstimate = count(par%toEstimate)

        ! Close file
        close (unit)

    end subroutine paramLoadStarting


    !
    ! Save all WL parameters to text file suitable for reading in next time
    !
    subroutine paramSaveWL(par,paramFile,headerText)
        implicit none
        type(param_t), intent(in) :: par
        character(*),  intent(in) :: paramFile
        character(*),  intent(in) :: headerText
        integer                   :: paramUnit, iParam


        ! Get unit, open it and write header
        paramUnit = 946
        open (paramUnit, file=paramFile, action='write')
        write(paramUnit, '("! ",A)') headerText

        ! Write parameters

        write(paramUnit, '(/,A)') '! Detrended gross monthly earnings in 1000s pounds: by gender, ability, educ (g,theta,s) '
        do iParam=1, 12
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! coeffs on age in log wage eq: by gender and education (g,s) '
        do iParam=13, 30
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! Variance of innovation in log productivity (xi): by gender and education (g,s) '
        do iParam=31, 36
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! Variance of measurement error in earnings (me): by gender and education (g,s)'
        do iParam=37, 42
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! autocorrelation in unobserv log productivity: by gender and education (g,s) '
        do iParam=43, 48
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! Pref for not working in C: intercept by gender, preferences and education (g,s) '
        do iParam=49, 54
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! Pref for not working in S: intercept by gender, preferences and education (g,s) '
        do iParam=55, 60
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! Unobserved heterogeneity in preferences (g,s) '
        do iParam=61, 66
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! coeffs on age in pref for working in C: by gender and education (g,s) '
        do iParam=67, 84
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! coeffs on age in pref for working in S: by gender and education (g,s) '
        do iParam=85, 102
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! Variance in preferences for working: by gender and education (g,s) '
        do iParam=103, 108
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! weights in distribution of heterogeneity in preferences, eta (g,s) '
        do iParam=109, 114
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! weights in conditional distribution of heterogeneity in productivity (thm,thf,sm,sf): couples '
        do iParam=115, 141
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! weights in conditional distribution of heterogeneity in productivity (g,s): singles '
        do iParam=142, 147
            write(paramUnit, 316) par%name(iParam), par%val(iParam), par%lb(iParam), par%ub(iParam), par%toEstimate(iParam)
        end do

        ! Close file
        close (paramUnit)
        
        316 format (T1, A15, T17, F15.8, T35, F15.8, T53, F15.8, T71, L1)
        
    end subroutine paramSaveWL


    !
    ! Save all SpH parameters to text file suitable for reading in next time
    !
    subroutine paramSaveSpH(paramFile,headerText)
        implicit none
        character(*),  intent(in) :: paramFile
        character(*),  intent(in) :: headerText
        integer                   :: paramUnit, iParam, i


        ! Get unit, open it and write header
        paramUnit = 523
        open (paramUnit, file=paramFile, action='write')
        write(paramUnit, '("! ",A)') headerText

        ! Write parameters
        write(paramUnit, '(/,A)') '! coeffs determining the utility cost of education costS(g,s,th) '
            do iParam=1, 8
            write(paramUnit, 316) paramS%name(iParam), paramS%val(iParam), paramS%lb(iParam), paramS%ub(iParam), paramS%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! value of HC in marriage, men - mxU(m,Hm,Hf) = exp(betacoeff*Vm(m,Hm,Hf))'
        do iParam=9, 44
            write(paramUnit, 316) paramS%name(iParam), paramS%val(iParam), paramS%lb(iParam), paramS%ub(iParam), paramS%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! preferences for remaining single prefC0(g,th,s) '
        do iParam=45, 56
            write(paramUnit, 316) paramS%name(iParam), paramS%val(iParam), paramS%lb(iParam), paramS%ub(iParam), paramS%toEstimate(iParam)
        end do

        write(paramUnit, '(/,A)') '! pref for similarly educated partners prefC1(g, th, d1/2: 1/2 levels diff)'
        do iParam=57, 64
            write(paramUnit, 316) paramS%name(iParam), paramS%val(iParam), paramS%lb(iParam), paramS%ub(iParam), paramS%toEstimate(iParam)
        end do

        ! Close file
        close (paramUnit)
        
        316 format (T1, A15, T17, F15.8, T35, F15.8, T53, F15.8, T71, L1)
        
    end subroutine paramSaveSpH


    !
    ! Print parameter estimates, SE and VarCov matrix to files  
    !      
    subroutine PrintEstimates(fileP,param)
        implicit none
        character(*), intent(in)  :: fileP
        type(param_t), intent(in) :: param
        integer                   :: unit, n, i

        unit = 7942
        open (unit, file=fileP, action='write')
        write(unit, '("                      coeff      st err    coeff/se")')
        n = 0
        do i = 1,param%n
            if (param%toEstimate(i) == .true.) then
                write(unit, 316) param%name(i), param%val(i), param%se(i), param%val(i)/param%se(i)
            end if
        end do
        close (unit)
        316 format (A15,3F12.5)

    end subroutine PrintEstimates



    ! ----------------------------------------------------------------------------------------------------------------
    ! Moments
    !

    !
    ! Read data moments for WL problem
    !
    subroutine loadDataMomWL(momWL)
        implicit none
        type(momWL_t),  intent(inout) :: momWL
        
        call LoadMomFromFile(momPath//'EregrM.txt',        len(momPath//'EregrM.txt'), momWL%Eregr(1),               0)
        call LoadMomFromFile(momPath//'EregrW.txt',        len(momPath//'EregrW.txt'), momWL%Eregr(2),               0)
        call LoadMomFromFile(momPath//'EdistM.txt',        len(momPath//'EdistM.txt'), momWL%Edist(1),               0)
        call LoadMomFromFile(momPath//'EdistW.txt',        len(momPath//'EdistW.txt'), momWL%Edist(2),               0)
        call LoadMomFromFile(momPath//'lWdistCm.txt',      len(momPath//'lWdistCm.txt'), momWL%lWdistC(1),           0)
        call LoadMomFromFile(momPath//'lWdistCw.txt',      len(momPath//'lWdistCw.txt'), momWL%lWdistC(2),           0)
        call LoadMomFromFile(momPath//'lWdistSm.txt',      len(momPath//'lWdistSm.txt'), momWL%lWdistS(1),           0)
        call LoadMomFromFile(momPath//'lWdistSw.txt',      len(momPath//'lWdistSw.txt'), momWL%lWdistS(2),           0)
        call LoadMomFromFile(momPath//'lWdistCm_smsw.txt', len(momPath//'lWdistCm_smsw.txt'), momWL%lWdistC_smsf(1), 0)
        call LoadMomFromFile(momPath//'lWdistCw_smsw.txt', len(momPath//'lWdistCw_smsw.txt'), momWL%lWdistC_smsf(2), 0)
        call LoadMomFromFile(momPath//'lWjointC.txt',      len(momPath//'lWjointC.txt'), momWL%lWjointC,             0)
        call LoadMomFromFile(momPath//'lWresm.txt',        len(momPath//'lWresm.txt'), momWL%lWres(1),               0)
        call LoadMomFromFile(momPath//'lWresw.txt',        len(momPath//'lWresw.txt'), momWL%lWres(2),               0)

    end subroutine loadDataMomWL


    !
    ! Read data moments for education and sorting problem
    ! 
    subroutine loadDataMomSpH(momSpH)
        implicit none
        type(momSpH_t), intent(inout) :: momSpH
        integer            :: sm,sf
        
        call LoadMomFromFile(momPath//'regrSm.txt',   len(momPath//'regrSm.txt'),   momSpH%regrS(1), 1)
        call LoadMomFromFile(momPath//'regrSw.txt',   len(momPath//'regrSw.txt'),   momSpH%regrS(2), 1)
        call LoadMomFromFile(momPath//'probSm.txt',   len(momPath//'probSm.txt'),   momSpH%probS(1), 0)
        call LoadMomFromFile(momPath//'probSw.txt',   len(momPath//'probSw.txt'),   momSpH%probS(2), 0)
        call LoadMomFromFile(momPath//'probSmSw.txt', len(momPath//'probSmSw.txt'), momSpH%probSmSw, 0)
                
    end subroutine loadDataMomSpH


    !
    ! Read matching patterns by education
    !
    subroutine loadMatchByS
        implicit none
        integer       :: unit, sm, sf
        real(wp)      :: se1, se2, se3
        character(12) :: label


        Psmsf     = 0.0_wp
        Psmsf_se  = 0.0_wp
        unit  = 6536
        open(unit, file = momPath//'probSmSw.txt', action = 'read')
        read(unit, 656) label
        read(unit, 543) label,  Psmsf(0,1), se1, Psmsf(0,2), se2, Psmsf(0,3), se3
        read(unit, 543) label,  Psmsf(1,1), Psmsf_se(1,1,1) , Psmsf(1,2), Psmsf_se(1,1,2), Psmsf(1,3), Psmsf_se(1,1,3)
        read(unit, 543) label,  Psmsf(2,1), Psmsf_se(1,2,1) , Psmsf(2,2), Psmsf_se(1,2,2), Psmsf(2,3), Psmsf_se(1,2,3)
        read(unit, 543) label,  Psmsf(3,1), Psmsf_se(1,3,1) , Psmsf(3,2), Psmsf_se(1,3,2), Psmsf(3,3), Psmsf_se(1,3,3)
        read(unit, 543) label,  Psmsf(1,0), se1, Psmsf(2,0), se2, Psmsf(3,0), se3
        Psmsf_se(2,:,:) = Psmsf_se(1,:,:)
        close(unit)
        
        656 format (A12,6F13.7)
        543 format (A12,6F13.7)

    end subroutine loadMatchByS


    !
    ! General routine: load data moments from file
    ! 
    subroutine LoadMomFromFile(filename, nchar, mom, dregr)
        implicit none
        integer, intent(in)         :: nchar, dregr
        character(nchar),intent(in) :: filename
        type(mom_t),intent(out)     :: mom
        integer                     :: unit, ios, s

        ! Initialise mom
        mom%n        = 1
        mom%name     = ''
        mom%dta      = 0.0_wp
        mom%dtaQ     = 0.0_wp
        mom%se       = 0.0_wp
        mom%sim      = 0.0_wp
        mom%sediff   = 0.0_wp
        mom%ssdiff   = 0.0_wp

        ! open file containing moments info
        unit = 4
        open(unit, file=filename, action='read')
        
        ! Read lines
        do
            if (dregr==0) then
                read(unit, 567, iostat=ios) mom%name(mom%n),  mom%dta(mom%n,1), mom%se(mom%n,1), &
                                            mom%dta(mom%n,2), mom%se(mom%n,2),  mom%dta(mom%n,3), mom%se(mom%n,3)
            else if (dregr==1) then
                read(unit, 234, iostat=ios) mom%name(mom%n),  mom%dta(mom%n,1), mom%se(mom%n,1), &
                                            mom%dta(mom%n,2), mom%se(mom%n,2),  mom%dta(mom%n,3), mom%se(mom%n,3)
            end if
                                            
            if (mom%name(mom%n)(1:1)=="Q") then
            do s=1,3
                mom%dtaQ(mom%n,s) = mom%dta(mom%n,s)
                if (mom%name(mom%n)(2:3)=="10") mom%dta(mom%n,s) = .10_wp 
                if (mom%name(mom%n)(2:3)=="25") mom%dta(mom%n,s) = .25_wp 
                if (mom%name(mom%n)(2:3)=="50") mom%dta(mom%n,s) = .50_wp 
                if (mom%name(mom%n)(2:3)=="75") mom%dta(mom%n,s) = .75_wp 
                if (mom%name(mom%n)(2:3)=="90") mom%dta(mom%n,s) = .90_wp 
            end do
            end if
                                        
            ! Exit do loop if end of file
            if (ios < 0) then
                mom%n = mom%n - 1
                exit
            end if

            ! If not blank lines/comments...
            if ((mom%name(mom%n)(1:1) .ne. " ") .and. (mom%name(mom%n)(1:1) .ne. "!")) then
                mom%n = mom%n + 1
            end if

        end do

        close (unit)
        567 format (A12,6F13.7)
        234 format (A20,6F13.3)

    end subroutine LoadMomFromFile


    !
    ! print SpH moments to file
    !
    subroutine writeSimMomSpH(momSpH, filename)
        implicit none
        type(momSpH_t), intent(in) :: momSpH 
        character(*), intent(in)   :: filename
        integer                    :: unit

        unit = 899
        open(unit, file=filename, action='write', status='replace')
        call PrintMomentsSpH(momSpH, unit, len('Matching patterns'), 'Matching patterns')
        call PrintMoments(momSpH%probS(1), unit, len('prob(S): men'),   'prob(S): men')
        call PrintMoments(momSpH%probS(2), unit, len('prob(S): women'), 'prob(S): women')
        call PrintMoments(momSpH%regrS(1), unit, len('regr(S): men'),   'regr(S): men')
        call PrintMoments(momSpH%regrS(2), unit, len('regr(S): women'), 'regr(S): women')
        close (unit)

    end subroutine writeSimMomSpH


    !
    ! Print data and simulated moments to unit, latex format
    !
    subroutine PrintMomentsSpH(momSpH, unit, nchar, title)
        implicit none
        type(momSpH_t), intent(in)   :: momSpH 
        integer,intent(in)           :: unit, nchar
        character(nchar), intent(in) :: title
        integer                      :: mth, ms, mh, wth, ws, wh, f, th, s, pth, h, ps, ph
        real(wp)                     :: Psmsf_dta(ng,ns,ns)


        write(unit,*)
        write(unit, '( "\begin{table}[htbp]\centering"                            )')
        write(unit, '( "\caption{", A40, "}"                                      )') title
        write(unit, '( "\begin{tabular}{l*{4}{r}}"                                )')
        write(unit, '( "\hline\hline"                                             )')
        write(unit, '( A25, " & ", A15, " & ", A15, " & ", A15, " & ", A15, " \\" )') "Moment", "Data", "Simulated", "SE data", "No. SE diff"
        write(unit, '( "\hline\hline"                                             )')

        write(unit,'( "\multicolumn{5}{l}{single men, theta=1,2, s=1,2,3} \\" )')
        do mth = 1,ntheta
        do ms  = 1,ns
            wh=0
            write ( unit , '( " (g th s) = (m" , 2I2 , ") & " , F15.5 , " & " , F15.5 , " & " , F15.5 , " & " , F15.2 , " \\" )' ) &
                mth, ms, momSpH%probSpH(1,mth)%dta(wh+1,ms), momSpH%probSpH(1,mth)%sim(wh+1,ms), momSpH%probSpH(1,mth)%se(wh+1,ms), momSpH%probSpH(1,mth)%sediff(wh+1,ms)
        end do
        end do

        write(unit,'( "\multicolumn{5}{l}{married men, mh=1,...,6, wh=1,...,6} \\" )')
        do mth = 1,ntheta
        do ms  = 1,ns
            mh = (mth-1)*ns+ms
            do wh = 1,nH
                write(unit,'("(g mh wh) = (m", 2I2, ") & ", F15.5, " & ", F15.5, " & ", F15.5, " & ", F15.2, " \\" )' ) &
                    mh, wh, momSpH%probSpH(1,mth)%dta(wh+1,ms), momSpH%probSpH(1,mth)%sim(wh+1,ms), momSpH%probSpH(1,mth)%se(wh+1,ms), momSpH%probSpH(1,mth)%sediff(wh+1,ms)
            end do
        end do
        end do

        write(unit, '( "\hline" )')
        write(unit,*)

        write(unit,'( "\multicolumn{5}{l}{single women, theta=1,2, s=1,2,3} \\" )')
        do wth = 1,ntheta
        do ws  = 1,ns
            mh=0
            write(unit,'(" (g th s) = (w", 2I2, ") & ", F15.5, " & ", F15.5, " & ", F15.5, " & ", F15.2, " \\" )' ) &
                wth, ws, momSpH%probSpH(2,wth)%dta(mh+1,ws), momSpH%probSpH(2,wth)%sim(mh+1,ws), momSpH%probSpH(2,wth)%se(mh+1,ws), momSpH%probSpH(2,wth)%sediff(mh+1,ws)
        end do
        end do
        write(unit,*)
                    
        write(unit,'( "\multicolumn{5}{l}{married women, mh=1,...,6, wh=1,...,6} \\" )')
        do wth = 1,ntheta
        do ws  = 1,ns
            wh = (wth-1)*ns+ws
            do mh = 1,nH
                write(unit,'("(g mh wh) = (w", 2I2, ") & ", F15.5, " & ", F15.5, " & ", F15.5, " & ", F15.2, " \\" )' ) &
                    mh, wh, momSpH%probSpH(2,wth)%dta(mh+1,ws), momSpH%probSpH(2,wth)%sim(mh+1,ws), momSpH%probSpH(2,wth)%se(mh+1,ws), momSpH%probSpH(2,wth)%sediff(mh+1,ws)
            end do
        end do
        end do

        write(unit, '( "\hline\hline"  )')
        write(unit, '( "\end{tabular}" )')
        write(unit, '( "\end{table}"   )')

    end subroutine PrintMomentsSpH


    !
    ! General routine: print moments to unit, latex format
    ! 
    subroutine PrintMoments(mom,unit,nchar,title)
        implicit none
        type(mom_t),intent(in)      :: mom
        integer,intent(in)          :: unit, nchar
        character(nchar),intent(in) :: title
        integer                     :: i, s

        write(unit,*)
        write(unit, '( "\begin{table}[htbp]\centering"                            )')
        write(unit, '( "\caption{", A40, "}"                                      )') title
        write(unit, '( "\begin{tabular}{l*{4}{r}}"                                )')
        write(unit, '( "\hline\hline"                                             )')
        write(unit, '( A25, " & ", A15, " & ", A15, " & ", A15, " & ", A15, " \\" )') "Moment", "Data", "Simulated", "SE data", "No. SE diff"
        write(unit, '( "\hline\hline"                                             )')
        do s=1, ns
            write(unit,'( "\multicolumn{5}{l}{s = ", I1, "} \\" )') s
            do i=1, mom%n
                write(unit,'(A25, " & ", F15.5, " & ", F15.5, " & ", F15.5, " & ", F15.2, " \\" )' ) mom%name(i), mom%dta(i,s), mom%sim(i,s), mom%se(i,s), mom%sediff(i,s)
            end do
            write(unit, '( "\hline" )')
        end do
        write(unit, '( "Total sq differences      & \multicolumn{4}{r}{", F13.2, "} \\"   )') mom%ssdiff
        write(unit, '( "\hline\hline"                                                     )')
        write(unit, '( "\end{tabular}"                                                    )')
        write(unit, '( "\end{table}"                                                      )')
        
    end subroutine PrintMoments

    
    
    
    ! ----------------------------------------------------------------------------------------------------------------
    ! Data
    !

    !
    ! Load matrix from STATA
    !
    subroutine readMx(m, n, Mx, file, varname, l)
        implicit none
        integer, intent(in)      :: m, n, l
        real(wp), intent(out)    :: Mx(m,n) 
        character(*), intent(in) :: file
        character(l), intent(in) :: varname
        character(10)            :: var
        integer                  :: i, j, unit

        unit  = 688
        call openStata(file, unit)
        var(1:l) = varname
        do i=1,n
            j = 1 + int( floor( log10( real(i+tol,wp) ) ) )
            if (j==1) write(var(l+1:l+j), '(I1)') i
            if (j==2) write(var(l+1:l+j), '(I2)') i
            if (j==3) write(var(l+1:l+j), '(I3)') i
            if (j==4) write(var(l+1:l+j), '(I4)') i
            call readStata(Mx(:,i), var(1:l+j))
        end do
        call closeOpenStata() 

    end subroutine readMx


    !
    ! Print matrix to STATA
    !
    subroutine writeMx(m, n, Mx, file, varname, l)
        implicit none
        integer, intent(in)      :: m, n, l
        real(wp), intent(in)     :: Mx(m,n) 
        character(*), intent(in) :: file
        character(l), intent(in) :: varname
        character(10)            :: var
        integer                  :: i, j, k
        

        call saveStata(file, 3, m)
        var(1:l) = varname
        do i=1,n
            j = 1 + int( floor( log10( real(i+tol,wp) ) ) )
            if (j==1) write(var(l+1:l+j), '(I1)') i
            if (j==2) write(var(l+1:l+j), '(I2)') i
            if (j==3) write(var(l+1:l+j), '(I3)') i
            if (j==4) write(var(l+1:l+j), '(I4)') i
            call writeStata( (/ (Mx(k,i), k=1,m) /), var(1:l+j))
        end do
        call closeSaveStata

    end subroutine writeMx


    !
    ! Save sim structure to Stata
    ! 
    subroutine writeSimStata(file)
        implicit none
        character(*), intent(in) :: file 
        integer, parameter       :: tota = maxa-mina+1, ntypes = nH*neta
        integer                  :: i, a, rep, g
                
        call saveStata(file, 3, nobsLC*nrepsWL*tota)
        call writeStata( (/ ((  spread(i,                                 1,tota), i=1,nobsLC), rep=1,nrepsWL) /), 'i')
        call writeStata( (/ (   spread(rep,                        1,nobsLC*tota),              rep=1,nrepsWL) /), 'rep')
        call writeStata( (/ (( (a,                                   a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'a')
        call writeStata( (/ ((  spread(simLC(i)%coupleDta,                1,tota), i=1,nobsLC), rep=1,nrepsWL) /), 'couple')
        call writeStata( (/ ((  spread(simLC(i)%s(1),                     1,tota), i=1,nobsLC), rep=1,nrepsWL) /), 'sm')
        call writeStata( (/ ((  spread(simLC(i)%s(2),                     1,tota), i=1,nobsLC), rep=1,nrepsWL) /), 'sf')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%k(a),             a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'k')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%q(a),             a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'q')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%w(1,a),           a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'wm')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%w(2,a),           a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'wf')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%l(1,a),           a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'lm')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%l(2,a),           a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'lf')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%inc(a),           a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'y')
        call writeStata( (/ (( (simLC(i)%wl(1,rep)%insample(a),      a=mina,maxa), i=1,nobsLC), rep=1,nrepsWL) /), 'insample')
        call writeStata( (/ ((  spread(simLC(i)%insampleWL,               1,tota), i=1,nobsLC), rep=1,nrepsWL) /), 'insampleWL')
        call writeStata( (/ ((  spread(simLC(i)%insampleS,                1,tota), i=1,nobsLC), rep=1,nrepsWL) /), 'insampleEduc')
        call closeSaveStata
        
    end subroutine writeSimStata




    ! ----------------------------------------------------------------------------------------------------------------
    ! Results: print matching results to file/screen
    !

    !
    ! Print all results
    !
    subroutine writeResults(unit, file, sortingmx)
        implicit none
        integer, intent(in)      :: unit
        real(wp), intent(in)     :: sortingmx(ng,0:nH,0:nH)
        character(*), intent(in) :: file
        
        if (unit .NE. 6) open (unit, file=file, action='write')
        call writePth(unit)
        call writeGain( unit)
        call writeSurplus(unit)
        call writeSorting(unit,sortingmx)
        call writeSharing(unit)
        if (unit .NE. 6) close (unit)

    end subroutine writeResults


    !
    ! print sharing rule
    !
    subroutine writeSharing(unit)
        use parameters, only : shareM
        implicit none
        integer, intent(in) :: unit
        integer             :: mh
        
        write(unit,'(/, "sharing rule that clears market", /, "-------------------------------")') 
        write(unit,'("wh ", 6A9)') "wh1", "wh2", "wh3", "wh4", "wh5", "wh6"
        do mh=1,nH
            write(unit,'("mh", I1, 6F9.4)') mh, shareM(mh,1), shareM(mh,2), shareM(mh,3), shareM(mh,4), shareM(mh,5), shareM(mh,6)
        end do
    end subroutine writeSharing


    !
    ! print matching patterns
    !
    subroutine writeSorting(unit, sortingmx)
        implicit none
        integer, intent(in)  :: unit
        real(wp), intent(in) :: sortingmx(ng,0:nH,0:nH)
        integer              :: mh, ms, wh, ws
        
        write(unit,'(/, "matching patterns by HC", /, "-----------------------")') 
        write(unit,'("   ", 6A10)') "wh1", "wh2", "wh3", "wh4", "wh5", "wh6"
        do mh=1,nH
            write(unit,'("mh", I1, 6F10.3)') mh, (sum(sortingmx(:,mh,wh))/sum(sortingmx(:,1:nH,1:nH)), wh=1,6)
        end do

        write(unit,'(/, "joint probability of singlehood and HC", /, "--------------------------------------")') 
        write(unit,'("      ", 6A9)') "h1", "h2", "h3", "h4", "h5", "h6"
        write(unit,'("women ", 6F9.2)') (sortingmx(2,0,wh)*0.01_wp, wh=1,6)
        write(unit,'("men   ", 6F9.2)') (sortingmx(1,mh,0)*0.01_wp, mh=1,6)

        write(unit,'(/, "distribution of HC among singles", /, "--------------------------------")') 
        write(unit,'("      ", 6A9)') "h1", "h2", "h3", "h4", "h5", "h6"
        write(unit,'("women ", 6F9.2)') (sortingmx(2,0,wh)/sum(sortingmx(2,0,1:nH)), wh=1,6)
        write(unit,'("men   ", 6F9.2)') (sortingmx(1,mh,0)/sum(sortingmx(1,1:nH,0)), mh=1,6)

        write(unit,'(/, "matching patterns by Education: choice of men (lhs) and women (rhs)", /, &
                        "-------------------------------------------------------------------")') 
        write(unit,'("   ", 3A10)') "ws1", "ws2", "w33"
        do ms = 1,ns
            write(unit,'("ms", I1, 3F10.3)') ms, ( sum(sortingmx(:,ms,ws)+sortingmx(:,ms,ws+3)+sortingmx(:,ms+ns,ws)+sortingmx(:,ms+ns,ws+3))*0.5_wp , ws=1,3 )
        end do
        
    end subroutine writeSorting


    !
    ! print distribution of ability
    !
    subroutine writePth(unit)
        use parameters,  only : Pth
        implicit none
        integer, intent(in) :: unit
        
        write(unit, '(/, "Distribution of unobserved heterogeneity in HC: theta 1,2", /, "---------------------------------------------------------")') 
        write(unit, '("men   ",3F9.4)' ) Pth(1,1), Pth(1,2), sum(Pth(1,:))
        write(unit, '("women ",3F9.4)' ) Pth(2,1), Pth(2,2), sum(Pth(2,:))
    
    end subroutine writePth
    
    
    !
    ! print economic gains from marriage
    !
    subroutine writeGain(unit)
        use Vstartlife,  only : econgain
        implicit none
        integer, intent(in)  :: unit
        integer              :: i, j

        write(unit, '(/,"Economic gains Hm / Hf",/ , "----------------------")') 
        write(unit, '("  ",7I9)' ) 0, 1, 2, 3, 4, 5, 6
        do i=0,6
            write(unit, '(I2,7F9.2)' ) i, (econgain(i,j), j=0,6)
        end do

    end subroutine writeGain


    !
    ! print marital surplus
    !
    subroutine writeSurplus(unit)
        use Vstartlife,  only : surplus
        implicit none
        integer, intent(in)  :: unit
        integer              :: i

        write(unit,'(/, "Marital surplus Hm / Hf", /, "-----------------------")') 
        write(unit, '("  ",6I9)' ) 1, 2, 3, 4, 5, 6
        do i=1,6
            write(unit, '(I2,6F9.2)' ) i, surplus(i,1), surplus(i,2), surplus(i,3), surplus(i,4), surplus(i,5), surplus(i,6)
        end do

    end subroutine writeSurplus

end module IOroutines