! --------------------------------------------------------------------------------------------------------------------
! This file is part of the package EMar, developed to produce the results in the paper "The marriage market, labor 
!    supply and education choice," by P.A. Chiapppori, M. Costa-Dias and C. Meghir. 
! Copyright (c) 2017 Monica Costa Dias
!
! Routines to estimate parameters driving the choices of education and marital sorting that clear the marriage market 
!    and routines to bootstrap the st errors of the parameters' estimates
! --------------------------------------------------------------------------------------------------------------------


module estimationSpH

use paths
use precision,     only : wp, tol
use constants,     only : ntheta, ng, ns, nH
use parameters,    only : paramCopyS, param_t, paramS, nParamMax, shareM, Pth, PthS, PthC
use VstartLife,    only : econgain
use IOroutines,    only : writeResults, paramSaveSpH, PrintEstimates
use functions,     only : scaleToRange, realtostr
use estimationWL,  only : monitorQA, QAparameters
use solutionSpH,   only : SimMomentsSpH, SolveAndSimSpH, calcQformSpH, writeSimMomSpH, &
                          momSpH, bPth, bPthC, bPthS, bprobSpH, sortingmx
use stataMod

real(wp) :: QformScale

contains


    ! ----------------------------------------------------------------------------------------------------------------
    ! estimation routines
    !

    !
    ! Estimate parameters in SpH choice in 3 steps:
    ! 1. estimate without imposing market clearing or shutting down infeasible markets (where 1 spouse has 
    !    -inf utility)
    ! 2. impose market clearing
    ! 3. close infeasible markets (if any)
    !    
    subroutine estimateSpH (dU, dcostS, Qform)
        use nag_g_ib,  only : g05kbf, g05lgf
        implicit none
        integer, parameter   :: replic=50
        integer, intent(in)  :: dU, dcostS
        real(wp),intent(out) :: Qform
        integer              :: unit, checkMkt, mktclear, dS, dpH, violations, step, igen, iseed(4), &
                                ifail, unit1, i, j
        real(wp)             :: scaledQform, sharingmx(replic,(nH)*(nH)), Qmx(replic)


        ! solve and simulate LC choices, calculate simulated moments and value of objective function
        call SolveAndSimSpH(.false., ' ', .false., ' ', checkMkt, Qform)
        write( *, '(/,"    Initial Sum of Squared Differences in Moments: ", F15.6, /)') Qform

        ! estimate parameters without imposing market clearing and not shutting down markets
        checkMkt  = 0
        mktclear  = 0
        dS        = 1
        dpH       = 1
        call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)
        write(*,'( "  step 1 - Qform=", 1F12.5, /)') Qform

        ! calculate sharing rule imposing market clearing but not shutting down infeasible markets
        checkMkt  = 0
        mktclear  = 1
        dS        = 1
        dpH       = 1
        call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)
        write(*,'( "  step 2 - Qform=", 1F12.5, /)') Qform

        ! recalculate sharing rule that clears market, now imposing feasibility on all markets
        violations = sum( merge( 1, 0, shareM(:,:)<=0.0101_wp .or. shareM(:,:)>=0.9899_wp ) )
        if (violations>0) then
            checkMkt = 1
            call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)
            write(*,'( "  step 3 - Qform=", 1F12.5, /)') Qform
        end if

        ! save information
        call paramSaveSpH(paramPath//'endParamS.txt', 'Estimated parameters (Qform=' // realToStr(Qform) // ')')
        call SimMomentsSpH(Qform)
        write( *, '(/,"    Final Sum of Squared Differences in Moments: ", F15.6)') Qform
        call writeSimMomSpH(momSpH, mompath//'SimMomSpH.txt')
        call writeResults(7534, resultspath//'resultsSpH.txt', sortingmx)

    end subroutine estimateSpH




    ! ----------------------------------------------------------------------------------------------------------------
    ! objective function in estimation
    ! allows for 3 alternative estimation procedures:
    !    iuser(2)=1: estimate all parameters in SpH choice
    !    iuser(2)=2: estimate sharing rule only
    !    iuser(2)=3: estimate sharing rule only, conditional on education
    ! variable 'checkMkt' only relevant if mktclear==1
    ! mktclear=1 for estimating sharing rule only
    !
    subroutine objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)
        use nag_e_ib,  only : e04jcf
        implicit none
        integer,  intent(in)  :: mktclear, checkMkt, dS, dpH
        real(wp), intent(out) :: Qform
        integer               :: iuser(2), ifail, ncall, nb, ntoestimate, maxIter, mh
        real(wp)              :: scaledQform, ruser(1)


        ! parameters to estimate: if calculating sharing rule only
        if (mktclear==1) then
            call calcQformSpH(0, 1, checkMkt, dS, dpH, Qform)
            paramS%toEstimate(:)    = .false.          ! do not change parameters except for...
            paramS%toEstimate(9:44) = .true.           ! sharing rule
            
        ! parameters to estimate: set estimation conditions: if estimating all parameters in SpH choice
        else
            call calcQformSpH(1, 0, 0, 1, 1, Qform)
            paramS%toestimate(1:paramS%n) = .true.
        end if
        
        ! set estimating conditions
        call QAparameters(paramS)                           ! scale parameters
        call scaleToRange(Qform, QformScale, scaledQform)   ! scale objecive function
        Qform = Qform / (10.d0**QformScale)
        iuser(1) = merge(checkMkt, 0, mktclear==1)          ! whether markets with prices on the boundary are closed
        iuser(2) = merge(2+(1-dS), 1, mktclear==1)          ! estimation procedure
        
        ! estimation
        nb          = paramS%n
        ntoestimate = count(paramS%toEstimate)
        maxIter     = 100000
        ifail       = -1
        call e04jcf(objSpH, nb, 2*ntoestimate+1, paramS%b(1:nb), paramS%blb(1:nb), paramS%bub(1:nb), &
                    0.1_wp, tol, monitorQA, maxIter, Qform, ncall, iuser, ruser, ifail)

        ! Copy back from b vector and export parameters info
        paramS%val(1:nb) = paramS%b(1:nb) * paramS%bscale(1:nb) + paramS%bmean(1:nb)
        call paramCopyS(paramS%val(1:paramS%n), paramS%n, econgain)
        Qform = Qform * (10.d0**QformScale)
            
    end subroutine objcalcSpH


    !
    ! Objective function calculator
    !
    subroutine objSpH(nb, b, Qform, iuser, ruser, inform)
        implicit none
        integer, intent(in)     :: nb
        real(wp), intent(in)    :: b(nb)
        real(wp), intent(out)   :: Qform
        integer, intent(inout)  :: iuser(2)
        real(wp), intent(inout) :: ruser(*)
        integer,intent(out)     :: inform
        integer                 :: all, sharing, checkMkt, dS, dpH
        

        ! recover parameters in current iteration
        paramS%val(1:nb) = b(1:nb) * paramS%bscale(1:nb) + paramS%bmean(1:nb)

        ! calculate simulated moments & criterion        
        all      = merge(1, 0, iuser(2)==1)
        sharing  = merge(1, 0, iuser(2)>1)
        checkMkt = merge(1, 0, iuser(1)==1 .or. iuser(2)==3)
        dS       = merge(1, 0, iuser(2)<=2)
        dpH      = 1
        call calcQformSpH(all, sharing, checkMkt, dS, dpH, Qform)
        Qform = Qform / (10.d0**QformScale)
        
        ! this needs to be +ve for the estimation to continue
        inform = 1 

    end subroutine objSpH

    
    
    ! ----------------------------------------------------------------------------------------------------------------
    ! bootstrapping routines
    !

    !
    ! bootstrap SE of paratmeters driving education and marital choice
    !
    subroutine bootstrapSpH
        implicit none
        integer, parameter    :: bdraws=100
        integer               :: nb, iter, i, unit, violations, checkMkt, mktclear, dS, dpH
        real(wp)              :: scaledQform, Qform, bparam(0:bdraws,nParamMax), mean
        real(wp), allocatable ::  bprobS(:,:,:,:), bregrS(:,:,:,:)


        ! initialisation 
        ! store parameters, solve and simulated moments, calculate Qform and its scale
        nb             = paramS%n
        bparam         = 0.0_wp
        bparam(0,1:nb) = paramS%val(1:nb)
        call SolveAndSimSpH(.false., ' ', .false., ' ', 0, Qform)

        ! load BS moments
        if (allocated(bprobS)) deallocate(bprobS);   allocate ( bprobS(ng, bdraws, momSpH%probS(1)%n, ns) )
        if (allocated(bregrS)) deallocate(bregrS);   allocate ( bregrS(ng, bdraws, momSpH%regrS(1)%n, ns) )
        call loadBSdata(bdraws,  momSpH%probS(1)%n, momSpH%regrS(1)%n, bprobS, bregrS)

        ! now run BS iterations: estimate parameters for each draw of data+model moments
        do iter = 1,bdraws
        
            write(*,'(/,"iteration ", 1I4)') iter
            
            ! load moments for current iteration
            call loadBSreplic(iter, bdraws,  momSpH%probS(1)%n, momSpH%regrS(1)%n, bprobS, bregrS)            
            
            ! estimate parameters using same process as for main estimation
            ! start by not imposing market clearing
            checkMkt  = 0
            mktclear  = 0
            dS        = 1
            dpH       = 1
            call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)

            ! now calculate sharing rule that clears market 
            ! (without imposing feasibility, which is to say without imposing prices in admissible interval)
            checkMkt  = 0
            mktclear  = 1
            dS        = 1
            dpH       = 1
            call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)

            ! recalculate sharing rule that clears market, now imposing feasibility
            ! now impose both market clearing and feasibility
            violations = sum( merge( 1, 0, shareM(:,:)<=0.0101_wp .or. shareM(:,:)>=0.9899_wp ) )
            if (violations>0) then
                checkMkt = 1
                call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)
            end if

            ! store parameters for this replication and copy main estimates back into 'val' vector
            bparam(iter,1:nb) = paramS%b(1:nb) * paramS%bscale(1:nb) + paramS%bmean(1:nb)
            paramS%val(1:nb)  = bparam(0,1:nb)
        
        end do
        
        ! Calculate SEs   
        do i=1,nb
            mean         = sum(bparam(1:bdraws,i)) / real(bdraws,wp)
            paramS%se(i) = sqrt( sum( (bparam(1:bdraws,i)-mean)**2.0_wp )  / real(bdraws-1,wp) )
        end do
        
        ! Export parameters and SE
        paramS%toestimate(1:nb) = .true.
        paramS%val(1:nb)        = bparam(0,1:nb)
        call PrintEstimates(paramPath//'estimatesSpH.txt', paramS)
        
    end subroutine bootstrapSpH


    !
    ! load BS replications of estimating (data) moments
    !
    subroutine loadBSdata(bdraws, nprobS, nregrS, bprobS, bregrS)
        implicit none
        integer, intent(in)   :: bdraws, nprobS, nregrS
        real(wp), intent(out) :: bprobS(ng,bdraws,nprobS,ns), bregrS(ng,bdraws,nregrS,ns)
        integer               :: unit, g, s
        character(10)         :: varname

        unit = 686
        call openStata(momPath//'b100_momentsSpH.dta', unit)
        do g=1,ng
        do s=1,ns
            if (g==1) write (varname(1:3), '("pm",I1)') s
            if (g==2) write (varname(1:3), '("pw",I1)') s
            call readStata(bprobS(g,:,1,s), varname(1:3) )
            
            if (g==1) write (varname(1:6), '("rm",I1,"_ct")') s
            if (g==2) write (varname(1:6), '("rw",I1,"_ct")') s
            call readStata(bregrS(g,:,1,s), varname(1:6) )
            
            if (g==1) write (varname(1:6), '("rm",I1,"_pY")') s
            if (g==2) write (varname(1:6), '("rw",I1,"_pY")') s
            call readStata(bregrS(g,:,2,s), varname(1:6) )
            
            if (g==1) write (varname(1:7), '("rm",I1,"_pY2")') s
            if (g==2) write (varname(1:7), '("rw",I1,"_pY2")') s
            call readStata(bregrS(g,:,3,s), varname(1:6) )
        end do
        end do
        call closeOpenStata()
             
    end subroutine loadBSdata


    !
    ! load moments for current BS iteration
    !
    subroutine loadBSreplic(i, bdraws, nprobS, nregrS, bprobS, bregrS)
        implicit none
        integer, intent(in)  :: i, bdraws, nprobS, nregrS
        real(wp), intent(in) :: bprobS(ng,bdraws,nprobS,ns), bregrS(ng,bdraws,nregrS,ns)
        integer              :: g, s, th, ph
        
        forall (g=1:ng, th=1:ntheta, s=1:ns, ph=0:nH) momSpH%probSpH(g,th)%dta(ph+1,s) = bprobSpH(g,th,s,ph,i)
        
        do g=1,ng
            momSPH%probS(g)%dta(1,:) = bprobS(g,i,1,:)
            momSPH%regrS(g)%dta(1,:) = bregrS(g,i,1,:)
            momSPH%regrS(g)%dta(2,:) = bregrS(g,i,2,:)
            momSPH%regrS(g)%dta(3,:) = bregrS(g,i,3,:)
        end do
        
        ! and other aggregate moments needed in simulations
        Pth  = bPth(i,:,:)
        PthC = bPthC(i,:,:,:,:)
        PthS = bPthS(i,:,:,:)
        
    end subroutine loadBSreplic

end module estimationSpH

