! --------------------------------------------------------------------------------------------------------------------
! This file is part of the package EMar, developed to produce the results in the paper "The marriage market, labor 
!    supply and education choice," by P.A. Chiapppori, M. Costa-Dias and C. Meghir. 
! Copyright (c) 2017 Monica Costa Dias
!
! Routines to calculate moments using simulated data and compute total of the squared differences between
!     data and model moments; WL period
! --------------------------------------------------------------------------------------------------------------------


module momentsWL

use precision,    only : wp, tol
use constants,    only : ns, ng, penalty
use parameters,   only : param_t, paramWL, seME
use SimStructure, only : simLC, nobsLC, nrepsWL
use momData,      only : momWL_t, nmom
use IOroutines,   only : loadDataMomWL, PrintMoments
use stataMod

implicit none

real(wp), parameter, dimension(ns), private :: p30m = (/ 2.85635_wp, 2.76377_wp, 2.86265_wp /), &
                                               p70m = (/ 3.25134_wp, 3.13636_wp, 3.17491_wp /), &
                                               p30f = (/ 2.28289_wp, 2.47420_wp, 2.86757_wp /), &
                                               p70f = (/ 2.91657_wp, 3.10725_wp, 3.41765_wp /)

real(wp), private, target, allocatable, dimension(:)     :: d25to35, wgt, insampleC 
real(wp), private, target, allocatable, dimension(:,:)   :: X, ME, dEC, dES, lWres, lWresC, lWresC2, lWresS, lWresS2, LTempC, &
                                                            LTempS, insampleS, insampleLTempC, insampleLTempS
real(wp), private, target, allocatable, dimension(:,:,:) :: dSC, LTsC, dSS, LTsS, insample, dE, dS, clWres
real(wp), pointer                                        :: pX(:,:), py(:)

integer,  private            :: totn, nlags
integer,  parameter, private :: nX=3

contains



    !-----------------------------------------------------------------------------------------------------------------
    ! Calculate all moments used to estimate parameters in WL problem, based on simulated data 
    ! Then calculate value of minimising criterion
    !
    
    subroutine SimMomentsWL(printfile, file, Qform, mom)
        use nag_g_ib,  only : g05kbf, g05laf
        implicit none
        logical,intent(in)            :: printfile
        character(*),intent(in)       :: file
        type(momWL_t), intent(out)    :: mom
        real(wp), intent(out)         :: Qform
        integer                       :: s, ss, sm, sf, g, sg, imom, i, nb, inX(nX), ifail, lag, unit
        real(wp)                      :: x1(nmom), x2, EinC, NinC, EinS, NinS, p
        real(wp), allocatable         :: select(:)


        ! Import data moments
        call loadDataMomWL(mom)


        ! Prepare simulated data
        call getRegrMtrx
        if (allocated(select)) deallocate(select);   allocate (select(totn))
        
        
        ! Earnings in couples, not conditional on spouse's education
        do s = 1,ns
        do g = 1,ng
        
            x1 = 0.0_wp
            x2 = 0.0_wp
            
            ! calculate sums for moments
            select(:) = insampleC(:) * d25to35(:) * dEC(:,g) * dSC(:,g,s)   ! select sample
            x2    = sum( select(:) )                                        ! nr observations in relevant sample
            x1(1) = sum( select(:) * lWresC(:,g) )                          ! mean
            x1(2) = sum( select(:) * lWresC2(:,g) )                         ! var
            do imom=3,7                                                     ! Q10, 25, 50, 75, 90
                x1(imom) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,g)<=mom%lWdistC(g)%dtaQ(imom,s)) )
            end do

            ! calculate simulated moments and store 
            call getRatio(x1(1), x2, mom%lWdistC(g)%dta(1,s), mom%lWdistC(g)%se(1,s), mom%lWdistC(g)%sim(1,s), mom%lWdistC(g)%sediff(1,s))
            x1(2) = x1(2) - x2*(mom%lWdistC(g)%sim(1,s)**2)
            do imom=2,mom%lWdistC(g)%n
                call getRatio(x1(imom), x2, mom%lWdistC(g)%dta(imom,s), mom%lWdistC(g)%se(imom,s), mom%lWdistC(g)%sim(imom,s), mom%lWdistC(g)%sediff(imom,s))
            end do
            
        end do
        end do


        ! Earnings among singles
        do s = 1,ns
        do g = 1,ng
        
            x1 = 0.0_wp
            x2 = 0.0_wp
            
            ! calculate sums for moments
            select(:) = insampleS(:,g) * d25to35(:) * dES(:,g) * dSS(:,g,s)   ! select sample
            x2    = sum( select(:) )                                          ! nr observations in relevant sample
            x1(1) = sum( select(:) * lWresS(:,g) )                            ! mean
            x1(2) = sum( select(:) * lWresS2(:,g) )                           ! var
            do imom=3,7                                                       ! Q10, 25, 50, 75, 90
                x1(imom) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresS(:,g)<=mom%lWdistS(g)%dtaQ(imom,s)) )
            end do

            ! calculate simulated moments and store 
            call getRatio(x1(1), x2, mom%lWdistS(g)%dta(1,s), mom%lWdistS(g)%se(1,s), mom%lWdistS(g)%sim(1,s), mom%lWdistS(g)%sediff(1,s))
            x1(2) = x1(2) - x2*(mom%lWdistS(g)%sim(1,s)**2)
            do imom=2,mom%lWdistS(g)%n
                call getRatio(x1(imom), x2, mom%lWdistS(g)%dta(imom,s), mom%lWdistS(g)%se(imom,s), mom%lWdistS(g)%sim(imom,s), mom%lWdistS(g)%sediff(imom,s))
            end do

        end do
        end do
        

        !  Earnings in couples, conditional on spouse's education
        do g = 1,ng
        do s = 1,ns

            x1 = 0.0_wp
            x2 = 0.0_wp
            sg = 3-g

            do ss = 1,ns

                ! calculate sums for moments
                select(:) = insampleC(:) * dEC(:,g) * dSC(:,g,s) * dSC(:,sg,ss)   ! select sample
                x2    = sum( select(:) )                                          ! nr observations in relevant sample
                x1(1) = sum( select(:) * lWresC(:,g) )                            ! mean
                x1(2) = sum( select(:) * lWresC2(:,g) )                           ! var

                ! calculate simulated moments and store 
                imom = ss
                call getRatio(x1(1), x2, mom%lWdistC_smsf(g)%dta(imom,s), mom%lWdistC_smsf(g)%se(imom,s), mom%lWdistC_smsf(g)%sim(imom,s), mom%lWdistC_smsf(g)%sediff(imom,s))
                x1(2) = x1(2) - x2*(mom%lWdistC_smsf(g)%sim(imom,s)**2)
                call getRatio(x1(2), x2, mom%lWdistC_smsf(g)%dta(imom+ns,s), mom%lWdistC_smsf(g)%se(imom+ns,s), mom%lWdistC_smsf(g)%sim(imom+ns,s), mom%lWdistC_smsf(g)%sediff(imom+ns,s))

            end do

        end do
        end do


        ! Joint distribution of earnings in couples
        do sm = 1,ns
        do sf = 1,ns

            ! compute sums for moments
            x1 = 0.0_wp
            x2 = 0.0_wp

            ! calculate sums for moments
            select(:) = insampleC(:) * dEC(:,1) * dEC(:,2) * dSC(:,1,sm) * dSC(:,2,sf)    ! select sample: couples (sm,sf) where both spouses work
            x2        = sum( select(:) )                                                  ! nr observations in relevant sample

            x1(1) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)<=p30m(sm) .and. lWresC(:,2)<=p30f(sf)) )                               ! man in bottom tercile, woman in terciles 1, 2, 3
            x1(2) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)<=p30m(sm) .and. lWresC(:,2)> p30f(sf) .and. lWresC(:,2)<=p70f(sf)) )
            x1(3) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)<=p30m(sm) .and. lWresC(:,2)> p70f(sf)) )

            x1(4) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)>p30m(sm) .and. lWresC(:,1)<=p70m(sm) .and. lWresC(:,2)<=p30f(sf)) )    ! man in middle tercile, woman in terciles 1, 2, 3
            x1(5) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)>p30m(sm) .and. lWresC(:,1)<=p70m(sm) .and. lWresC(:,2)> p30f(sf) .and. lWresC(:,2)<=p70f(sf)) )
            x1(6) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)>p30m(sm) .and. lWresC(:,1)<=p70m(sm) .and. lWresC(:,2)> p70f(sf)) )

            x1(7) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)>p70m(sm) .and. lWresC(:,2)<=p30f(sf)) )                                ! man in top tercile, woman in terciles 1, 2, 3
            x1(8) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)>p70m(sm) .and. lWresC(:,2)> p30f(sf) .and. lWresC(:,2)<=p70f(sf)) )
            x1(9) = sum( select(:) * merge(1.0_wp, 0.0_wp, lWresC(:,1)>p70m(sm) .and. lWresC(:,2)> p70f(sf)) )

            ! calculate simulated moments and store 
            do i=1,9
                imom = (sm-1)*9+i
                call getRatio(x1(i), x2, mom%lWjointC%dta(imom,sf), mom%lWjointC%se(imom,sf), mom%lWjointC%sim(imom,sf), mom%lWjointC%sediff(imom,sf))
            end do

        end do
        end do
        
        
        ! Autocorrelation of the residuals in the earnings regression
        do s = 1,ns
        do g = 1,ng
            x1 = 0.0_wp
            x2 = 0.0_wp
            do lag=0,nlags
                x2    = sum( insample(:,g,0) * insample(:,g,lag) * dE(:,g,0) * dE(:,g,lag) * dS(:,g,s) )
                x1(1) = sum( insample(:,g,0) * insample(:,g,lag) * dE(:,g,0) * dE(:,g,lag) * dS(:,g,s) * clWres(:,g,0) * clWres(:,g,lag) )
                call getRatio(x1(1), x2, mom%lWres(g)%dta(lag+1,s), mom%lWres(g)%se(lag+1,s), mom%lWres(g)%sim(lag+1,s), mom%lWres(g)%sediff(lag+1,s))
            end do
        end do
        end do


        ! Employment by marital status
        do g = 1,ng
        do s = 1,ns

            ! regressions: couples
            nb       = 4                           ! nr regressors, including intercept
            inX(1:3) = 1                           ! columns in X to include as covariates
            pX       => X                          ! covariates
            py       => dEC(:,g)                   ! dependent variable
            wgt(:)   = insampleC(:) * dSC(:,g,s)   ! observations in regression
            ifail    = 0
            call OLS(nX, inX, totn, 1, nb, mom%Eregr(g)%sim(1:nb,s), ifail )
            if (ifail==0) then
                do imom=1,nb
                    mom%Eregr(g)%sediff(imom,s) = abs(mom%Eregr(g)%sim(imom,s)-mom%Eregr(g)%dta(imom,s)) / mom%Eregr(g)%se(imom,s)
                end do
            else
                mom%Eregr(g)%sim(1:nb,s)    = penalty
                mom%Eregr(g)%sediff(1:nb,s) = penalty
            end if
            
            ! regressions: singles
            py     => dES(:,g)                     ! dependent variable
            wgt(:) = insampleS(:,g) * dSS(:,g,s)   ! observations in regression
            ifail  = 0
            call OLS(nX, inX, totn, 1, nb, mom%Eregr(g)%sim(nb+1:2*nb,s), ifail )
            if (ifail==0) then
                do imom=nb+1,2*nb
                    mom%Eregr(g)%sediff(imom,s) = abs(mom%Eregr(g)%sim(imom,s)-mom%Eregr(g)%dta(imom,s)) / mom%Eregr(g)%se(imom,s)
                end do
            else
                mom%Eregr(g)%sim(nb+1:2*nb,s)    = penalty
                mom%Eregr(g)%sediff(nb+1:2*nb,s) = penalty
            end if
            
            ! distribution of LC employment: couples
            imom = 0
            do i=1,3
                imom  = imom+1
                p     = 0.2_wp+real(i,wp)*0.2_wp                                                        ! cutoff point: probability of employment
                x2    = sum( insampleLTempC(:,g)*LTsC(:,g,s) )                                          ! nr observations in relevant sample
                x1(i) = sum( insampleLTempC(:,g)*LTsC(:,g,s)*merge(1.0_wp,0.0_wp,LTempC(:,g)<p+tol) )   ! nr observations below cutoff point
                call getRatio(x1(i), x2, mom%Edist(g)%dta(imom,s), mom%Edist(g)%se(imom,s), mom%Edist(g)%sim(imom,s), mom%Edist(g)%sediff(imom,s))
            end do

            ! distribution of LC employment: singles
            do i=1,3
                imom = imom+1
                p     = 0.2_wp+real(i,wp)*0.2_wp                                                        ! cutoff point: probability of employment
                x2    = sum( insampleLTempS(:,g)*LTsS(:,g,s) )                                          ! nr observations in relevant sample
                x1(i) = sum( insampleLTempS(:,g)*LTsS(:,g,s)*merge(1.0_wp,0.0_wp,LTempS(:,g)<p+tol) )   ! nr observations below cutoff point
                call getRatio(x1(i), x2, mom%Edist(g)%dta(imom,s), mom%Edist(g)%se(imom,s), mom%Edist(g)%sim(imom,s), mom%Edist(g)%sediff(imom,s))
            end do

        end do
        end do
        

        ! Objective function: sum of squared differences
        mom%lWjointC%ssdiff = sum( mom%lWjointC%sediff(:,:)**2 )
        do g=1,ng
            mom%lWdistS(g)%ssdiff      = sum( mom%lWdistS(g)%sediff(:,:)**2 )
            mom%lWdistC(g)%ssdiff      = sum( mom%lWdistC(g)%sediff(:,:)**2 )
            mom%lWdistC_smsf(g)%ssdiff = sum( mom%lWdistC_smsf(g)%sediff(:,:)**2 )
            mom%lWres(g)%ssdiff        = sum( mom%lWres(g)%sediff(:,:)**2 )
            mom%Edist(g)%ssdiff        = sum( mom%Edist(g)%sediff(:,:)**2 )
            mom%Eregr(g)%ssdiff        = sum( mom%Eregr(g)%sediff(:,:)**2 )
        end do
        Qform = sum( mom%lWdistS(:)%ssdiff + mom%lWdistC(:)%ssdiff + mom%lWdistC_smsf(:)%ssdiff + &
                     mom%lWres(:)%ssdiff   + mom%Eregr(:)%ssdiff   + mom%Edist(:)%ssdiff )      + mom%lWjointC%ssdiff


        ! save moments to file
        if (printfile==.true.) then
            unit=738
            open (unit, FILE = file)
            call PrintMoments(mom%lWdistS(1),      unit, 40, 'Residual lnY: single men by sm          ')
            call PrintMoments(mom%lWdistS(2),      unit, 40, 'Residual lnY: single women by sf        ')
            call PrintMoments(mom%lWdistC(1),      unit, 40, 'Residual lnY: married men by sm         ')
            call PrintMoments(mom%lWdistC(2),      unit, 40, 'Residual lnY: married women by sf       ')
            call PrintMoments(mom%lWdistC_smsf(1), unit, 40, 'Residual lnY: married men by sm x sf    ')
            call PrintMoments(mom%lWdistC_smsf(2), unit, 40, 'Residual lnY: married wommen by sm x sf ')
            call PrintMoments(mom%lWjointC,        unit, 40, 'Residual lnY: joint distr in couples    ')
            call PrintMoments(mom%lWres(1),        unit, 40, 'Residual lnY: autocorr, men by sm       ')
            call PrintMoments(mom%lWres(2),        unit, 40, 'Residual lnY: autocorr, women by sf     ')
            call PrintMoments(mom%Eregr(1),        unit, 40, 'Employment regressions: men by sm       ')
            call PrintMoments(mom%Eregr(2),        unit, 40, 'Employment regressions: women by sf     ')
            call PrintMoments(mom%Edist(1),        unit, 40, 'Employment distribution: men by sm      ')
            call PrintMoments(mom%Edist(2),        unit, 40, 'Employment distribution: women by sf    ')
            close(unit)
        end if
    
    end subroutine SimMomentsWL




    !-----------------------------------------------------------------------------------------------------------------
    ! Auxilliary functions
    !

    !
    ! Calculate contribution of moment measuring proportions to objective function 
    ! 
    subroutine getRatio(x1,x2,dtaM,seM,simM,sediff)
        implicit none
        real(wp), intent(in)  :: x1, x2, dtaM, seM
        real(wp), intent(out) :: simM,sediff

        if (x2>tol) then
            simM   = x1 / x2
            sediff = (abs(simM-dtaM)/seM)
        else
            simM   = penalty
            sediff = penalty
        end if

    end subroutine getRatio



    !
    ! OLS regression
    !
    subroutine OLS(nX, inX, nobs, intercept, nb, b, ifail)
        use nag_g_ib, only : G02DAF
        implicit none
        integer,intent(in)   :: nX, inX(nX), nobs, intercept, nb
        integer, intent(out) :: ifail
        real(wp),intent(out) :: b(nb)
        integer      :: df, rank
        real(wp)     :: rss, se(nb), cov(nb*(nb+1)/2), H(nobs), Q(nobs,nb+1), P(2*nb+nb*nb), wk(2*(nb-1)+nb*nb), res(nobs)
        logical      :: svd
        character(1) :: mean
        

        res = 0.0_wp

        if (sum(wgt(1:nobs)) > tol .and. sum(abs(wgt(1:nobs)*py(1:nobs))) > tol) then
            mean = 'M'
            if (intercept==0) mean = 'Z'
            ifail = -1
            call G02DAF(mean=mean,       weight='W', n=nobs,  X=pX,     ldX=nobs,       &
                        m=nX,            isX=inX,    ip=nb,   y=py,     wt=wgt(1:nobs), &
                        rss=rss,         idf=df,     b=b,     se=se,    cov=cov,        &
                        res=res(1:nobs), H=H,        Q=Q,     ldQ=nobs, svd=svd,        &
                        irank=rank,      P=P,        tol=tol, wk=wk,    ifail=ifail     )
        else 
            ifail = 1
        end if
       
    end subroutine OLS



    !
    ! Prepare simulated data to calculate moments
    !
    subroutine getRegrMtrx
        implicit none
        integer  :: lo, hi, rep, i, s, sm, sf, a, a1, aN, g, j, ifail, igen, iseed(4), lag
        real(wp) :: x1, x2, mean


        ! setup matrices
        a1    = 25
        aN    = 50
        totn  = nrepsWL*nobsLC*(aN-a1+1)
        nlags = 3
                
        if (allocated(d25to35))        deallocate(d25to35);         allocate (d25to35(totn))
        if (allocated(wgt))            deallocate(wgt);             allocate (wgt(totn))
        if (allocated(insample))       deallocate(insample);        allocate (insample(totn,ng,0:nlags))
        if (allocated(insampleC))      deallocate(insampleC);       allocate (insampleC(totn))
        if (allocated(insampleS))      deallocate(insampleS);       allocate (insampleS(totn,ng))
        if (allocated(insampleLTempC)) deallocate(insampleLTempC);  allocate (insampleLTempC(nobsLC*nrepsWL,ng))
        if (allocated(insampleLTempS)) deallocate(insampleLTempS);  allocate (insampleLTempS(nobsLC*nrepsWL,ng))
        if (allocated(X))              deallocate(X);               allocate (X(totn,nX))
        if (allocated(ME))             deallocate(ME);              allocate (ME(totn,ng))
        if (allocated(dE))             deallocate(dE);              allocate (dE(totn,ng,0:nlags))
        if (allocated(dEC))            deallocate(dEC);             allocate (dEC(totn,ng))
        if (allocated(dES))            deallocate(dES);             allocate (dES(totn,ng))
        if (allocated(lWres))          deallocate(lWres);           allocate (lWres(totn,ng))
        if (allocated(clWres))         deallocate(clWres);          allocate (clWres(totn,ng,0:nlags))
        if (allocated(lWresC))         deallocate(lWresC);          allocate (lWresC(totn,ng))
        if (allocated(lWresC2))        deallocate(lWresC2);         allocate (lWresC2(totn,ng))
        if (allocated(lWresS))         deallocate(lWresS);          allocate (lWresS(totn,ng))
        if (allocated(lWresS2))        deallocate(lWresS2);         allocate (lWresS2(totn,ng))
        if (allocated(dS))             deallocate(dS);              allocate (dS(totn,ng,ns))
        if (allocated(dSC))            deallocate(dSC);             allocate (dSC(totn,ng,ns))
        if (allocated(dSS))            deallocate(dSS);             allocate (dSS(totn,ng,ns))
        if (allocated(LTempC))         deallocate(LTempC);          allocate (LTempC(nobsLC*nrepsWL,ng))
        if (allocated(LTempS))         deallocate(LTempS);          allocate (LTempS(nobsLC*nrepsWL,ng))
        if (allocated(LTsC))           deallocate(LTsC);            allocate (LTsC(nobsLC*nrepsWL,ng,ns))
        if (allocated(LTsS))           deallocate(LTsS);            allocate (LTsS(nobsLC*nrepsWL,ng,ns))

        X              = 0.0_wp
        ME             = 0.0_wp
        dE             = 0.0_wp
        dEC            = 0.0_wp
        dES            = 0.0_wp
        lWres          = 0.0_wp
        clWres         = 0.0_wp
        lWresC         = 0.0_wp
        lWresC2        = 0.0_wp
        lWresS         = 0.0_wp
        lWresS2        = 0.0_wp
        dS             = 0.0_wp
        dSC            = 0.0_wp
        dSS            = 0.0_wp
        LTempC         = 0.0_wp
        LTempS         = 0.0_wp
        LTsC           = 0.0_wp
        LTsS           = 0.0_wp
        d25to35        = 0.0_wp
        insample       = 0.0_wp
        insampleC      = 0.0_wp
        insampleS      = 0.0_wp
        insampleLTempC = 0.0_wp
        insampleLTempS = 0.0_wp

        ! measurement error in earnings
        ifail = 0
        igen  = 0
        iseed = 0
        iseed(1) = 2762065
        call g05kbf(igen,iseed)
        do g=1,ng
            call g05laf (0.0_wp, 1.0_wp, totn, ME(:,g), igen, iseed, ifail)
        end do

        ! fill matrices with simulated data
        lo = 0
        hi = 0
        j  = 0
        do rep = 1,nrepsWL
        do i   = 1,nobsLC
        
            lo = hi+1            ! hi-lo+1 = length observ window - 1 (for 1st diff)
            hi = hi+(aN-a1+1)
            j  = j+1

            if (simLC(i)%insampleWL==1 .and. simLC(i)%age1<=aN .and. simLC(i)%ageN>=a1) then

                ! regressors
                forall (a=0:aN-a1) X(lo+a,1) = 0.1_wp*real(a,wp)
                X(lo:hi,2) = X(lo:hi,1)*X(lo:hi,1)
                X(lo:hi,3) = X(lo:hi,2)*X(lo:hi,1)

                ! sample selection
                insampleC(lo:hi)     = simLC(i)%wl(1,rep)%insample(a1:aN) * real(simLC(i)%coupleDta,wp)
                d25to35(lo:lo+35-a1) = 1.0_wp

                do g=1,ng
                    insampleS(lo:hi,g)    = simLC(i)%wl(1,rep)%insample(a1:aN) * merge(1.0_wp, 0.0_wp, simLC(i)%coupleDta==0 .and. simLC(i)%female==g-1)
                    insample(lo:hi,g,0)   = simLC(i)%wl(1,rep)%insample(a1:aN) * merge(1.0_wp, 0.0_wp, simLC(i)%coupleDta==1 .or. simLC(i)%female==g-1)

                    do s=1,ns
                        dS(lo:hi,g,s)  = merge(1.0_wp, 0.0_wp, simLC(i)%s(g)==s .and. (simLC(i)%coupleDta==1 .or. simLC(i)%female==g-1))
                        dSC(lo:hi,g,s) = merge(1.0_wp, 0.0_wp, simLC(i)%s(g)==s .and. simLC(i)%coupleDta==1)
                        dSS(lo:hi,g,s) = merge(1.0_wp, 0.0_wp, simLC(i)%s(g)==s .and. simLC(i)%coupleDta==0 .and. simLC(i)%female==g-1)
                    end do
                end do

                ! outcomes
                do g=1,ng
                if (simLC(i)%coupleDta==1 .or. simLC(i)%female==g-1) then
                
                    ! earnings net of age effects
                    lWres(lo:hi,g)  = ( simLC(i)%wl(1,rep)%lWres(g,a1:aN) + ME(lo:hi,g)*SEme(g,simLC(i)%s(g)) ) * merge(1.0_wp, 0.0_wp, simLC(i)%coupleDta==1 .or. simLC(i)%female==g-1)
                    lWresC(lo:hi,g) = lWres(lo:hi,g) * real(simLC(i)%coupleDta, wp)
                    lWresS(lo:hi,g) = lWres(lo:hi,g) * merge(1.0_wp, 0.0_wp, simLC(i)%coupleDta==0 .and. simLC(i)%female==g-1)
                    lWresC2(lo:hi,g) = lWresC(lo:hi,g)**2
                    lWresS2(lo:hi,g) = lWresS(lo:hi,g)**2

                    ! employment
                    dE(lo:hi,g,0) = merge( 1.0_wp, 0.0_wp, simLC(i)%wl(1,rep)%L(g,a1:aN)==0 .and. (simLC(i)%coupleDta==1 .or. simLC(i)%female==g-1) )
                    dEC(lo:hi,g)  = merge( 1.0_wp, 0.0_wp, simLC(i)%wl(1,rep)%L(g,a1:aN)==0 .and. simLC(i)%coupleDta==1 )
                    dES(lo:hi,g)  = merge( 1.0_wp, 0.0_wp, simLC(i)%wl(1,rep)%L(g,a1:aN)==0 .and. simLC(i)%coupleDta==0 .and. simLC(i)%female==g-1 )

                    ! employment over longer periods
                    if (sum(insampleC(lo:hi))>0.0_wp)   LTempC(j,g) = sum(dEC(lo:hi,g)*insampleC(lo:hi))   / sum(insampleC(lo:hi))
                    if (sum(insampleS(lo:hi,g))>0.0_wp) LTempS(j,g) = sum(dES(lo:hi,g)*insampleS(lo:hi,g)) / sum(insampleS(lo:hi,g))

                    insampleLTempC(j,g) = merge( 1.0_wp, 0.0_wp, sum(insampleC(lo:hi))>4.9_wp )
                    insampleLTempS(j,g) = merge( 1.0_wp, 0.0_wp, sum(insampleS(lo:hi,g))>4.9_wp )
                    
                    do s=1,ns
                        LTsC(j,g,s) = merge( 1.0_wp, 0.0_wp, simLC(i)%s(g)==s .and. simLC(i)%coupleDta==1 )
                        LTsS(j,g,s) = merge( 1.0_wp, 0.0_wp, simLC(i)%s(g)==s .and. simLC(i)%coupleDta==0 .and. simLC(i)%female==g-1 )
                    end do
                
                end if
                end do

            end if            
            
        end do
        end do
        

        ! log earnings net age effects, demeaned and lagged
        do g=1,ng
            do s=1,ns
                x1 = sum( insample(:,g,0) * dS(:,g,s) * dE(:,g,0) * lWres(:,g) )
                x2 = sum( insample(:,g,0) * dS(:,g,s) * dE(:,g,0) )
                mean = x1/x2
                clWres(:,g,0) = insample(:,g,0) * ( dS(:,g,s)*(lWres(:,g)-mean) + (1.0_wp-dS(:,g,s))*clWres(:,g,0) )
            end do
            
            do lag=1,3
                insample(1+lag:totn,g,lag) = insample(1:totn-lag,g,0)
                clWres(1+lag:totn,g,lag)   = clWres(1:totn-lag,g,0)
                dE(1+lag:totn,g,lag)       = dE(1:totn-lag,g,0)
            end do
        end do
        
    end subroutine getRegrMtrx

end module momentsWL


