! --------------------------------------------------------------------------------------------------------------------
! This file is part of the package EMar, developed to produce the results in the paper "The marriage market, labor 
!    supply and education choice," by P.A. Chiapppori, M. Costa-Dias and C. Meghir. 
! Copyright (c) 2017 Monica Costa Dias
!
! Data modules: define data used in all steps of solution, estimation and simulation as
!     well as some basic data routines
! --------------------------------------------------------------------------------------------------------------------



! --------------------------------------------------------------------------------------------------------------------
! Constant parameters
!

module precision
    integer, parameter   :: wp  = kind(1.0d0)
    real(wp), parameter  :: tol = 1.0e-6_wp
    real(wp), parameter  :: infty = 1.0e30_wp
    real(wp), parameter  :: neginfty = -1.0e30_wp
    integer, parameter   :: pi=3.1415926535898793238_wp
end module precision


module constants
    use precision, only :  wp
    integer,parameter   :: ng = 2                                 ! gender: 1=male, 2=female
    integer,parameter   :: ns = 3                                 ! education: 1=secondary, 2=HS, 3=Univ
    integer,parameter   :: ntheta = 2                             ! unobs heterogeneity in ability (theta)
    integer,parameter   :: nH=ns*ntheta
    integer,parameter   :: neta = 2                               ! unobs heterogeneity in preferences for working (eta)
    integer,parameter   :: mina = 23, maxa = 59, enda = 69        ! min and max age
    real(wp), parameter :: penalty = 50.0_wp                      ! penalty for picking parameters outside acceptable range
end module constants


module paths
    character(*), parameter  :: paramPath   = '../parameters/'
    character(*), parameter  :: momPath     = '../moments/'
    character(*), parameter  :: resultsPath = '../results/'
    character(*), parameter  :: dtaPath     = '../data/'
end module paths



! --------------------------------------------------------------------------------------------------------------------
! value of options at the start of working life
!
module VstartLife
    use precision, only : wp
    use constants, only : nH
    real(wp), dimension(0:nH,0:nH) :: upsilon, econgain   ! Evalue of HC in marriage/singlehood (upislo) and its exponential (econgain)
    real(wp), dimension(nH,nH)     :: surplus             ! surplus (excl random marital pref) ! TO DELETE??
end module VstartLife



! --------------------------------------------------------------------------------------------------------------------
! Moment structures and I/O routines
!
module momData

    use precision, only : wp
    use constants, only : ns, ng, ntheta
    
    integer, parameter  :: nmom   = 50      ! Total number of moments per section of Mstructure (upper bound)
    type mom_t
        integer         :: n                ! Number of moments
        character(50)   :: name(nmom)       ! Moments' names
        real(wp)        :: dta(nmom,ns)     ! Data moments
        real(wp)        :: dtaQ(nmom,ns)    ! Data quantile of the distribution: only to be used for quantile moments
        real(wp)        :: se(nmom,ns)      ! Data standard errors
        real(wp)        :: sim(nmom,ns)     ! Simulated moments
        real(wp)        :: sediff(nmom,ns)  ! no. se differences
        real(wp)        :: ssdiff           ! objective function: sum of squared se differences for set of moments
    end type mom_t

    type momWL_t
        type(mom_t)   :: lWdistS(ng)
        type(mom_t)   :: lWdistC(ng)
        type(mom_t)   :: lWdistC_smsf(ng)
        type(mom_t)   :: lWjointC
        type(mom_t)   :: lWres(ng)
        type(mom_t)   :: Edist(ng)
        type(mom_t)   :: Eregr(ng)
    end type momWL_t
    
    type momSpH_t
        type(mom_t) :: probSpH(ng,ntheta)   ! prob(S,pH | g, theta)
        type(mom_t) :: probS(ng)            ! prob(S | g, factors)
        type(mom_t) :: regrS(ng)            ! regression(S | g, factors)
        type(mom_t) :: probSmSw             ! joint distribution (Sm,Sw)
    end type momSpH_t

    real(wp) :: PSmSf(0:ns,0:ns), Psmsf_se(ng,ns,ns), Psmsf_sim(ng,ns,ns)

end module momData



! --------------------------------------------------------------------------------------------------------------------
! parameters (calibrated and to estimate)
!
module parameters

    use precision, only : wp
    use constants, only : ng, ns, ntheta, nH, maxa, mina, neta


    ! constants
   
    real(wp),parameter :: pQ   = 1.0_wp               !price of public good
    real(wp),parameter :: R    = 1.015_wp             !interest factor
    real(wp),parameter :: beta = 1.0_wp/1.02_wp       !discount factor 
    real(wp),parameter :: betacoef  = (1.0_wp-beta) / ( 1.0_wp - (beta**(maxa-mina+1)) )

    real(wp)           :: iBR                         !derived: (beta*R)^(-1)
    real(wp)           :: i2pQ, i4pQ, lnpQ            !derived: (2*pQ)^(-1) and (4*pQ)^(-1)

    real(wp),parameter :: taxC    = 0.15_wp           !tax rate, couples
    real(wp),parameter :: taxS    = 0.15_wp           !tax rate, singles
    real(wp),parameter :: transfC = 3.0_wp            !lump sum transfer to married couples per non-working spouse
    real(wp),parameter :: transfS = 3.0_wp            !lump sum transfer to singles  

    real(wp),parameter :: maxT=1.0_wp                 !upper bound on leisure time 
    real(wp)           :: CminC, QminC, CminS, QminS  !lower bound on private and public consumption, singles and couples
    

    ! parameters to estimate - post-marriage WL

    ! distribution of log wages over the LC 
    real(wp) :: meanlny_s1(ng), lny(ng,ntheta,ns), &                                                      ! detrended mean log earnings at 23 (1000 pounds)                
                pthC(ntheta,ntheta,ns,ns), pthS(ng,ntheta,ns), Pth(ng,ntheta), thetaval(ng,ntheta,ns), &  ! distribution of unobserved heterogeneity in wages
                returnS(ng,ns), &                                ! value of education in wages
                lny_a(ng,ns), lny_asq(ng,ns), lny_acb(ng,ns), &  ! age poly in wages (by gender and educ)
                Vxi(ng,ns), kappa(ng,ns), Vme(ng,ns)             ! earnings residual: var innov in productivty (Vxi), autocorrelation parameter (kappa), var ME (Vme)

    ! utility parameters
    real(wp) :: eta(neta,ng,ns), peta(ng,ns)                                    ! distribution of unobserved heterogeneity in preferences for working
    real(wp), dimension(ng,ns) :: alphaC0, alphaC_a, alphaC_asq, alphaC_acb, &  ! age poly in preferences (in couples, by gender & educ)
                                  alphaS0, alphaS_a, alphaS_asq, alphaS_acb, &  ! age poly in preferences (singles, by gender & educ)
                                  Vu                                            ! variance transitory preference shocks
   
    ! derived parameters
    real(wp), dimension(ng,ntheta,ns) :: lnyS, lnyC           ! net (of taxes) earnings rates
    real(wp), dimension(ng,ns)        :: SExi, SEe, SEu, SEme ! St errors of shocks
    real(wp), dimension(ng,ns)        :: Exi                  ! expected value of productivity shock (log-normal)


    ! parameters to estimate - education and sorting
        
    real(wp) :: mxU(ng,0:nH,0:nH)         ! exp EV human capital in marriage/singlehood - for indiv, including sharing
    real(wp) :: shareM(nH,nH)             ! sharing rule: % economic gains kept by men
    real(wp) :: prefC0(ng,ntheta,ns)      ! additive constant: preferences for staying single
    real(wp) :: prefC1(ng,ntheta,0:ns-1)  ! additive constant: preferences for marrying ps by (g,th)
    real(wp) :: costS(ng,ns,ntheta,0:1)   ! utility cost of education (ctt)
    real(wp) :: scaleU(ng)                ! scale of LT utility in choice of (S,pH)
    real(wp) :: SEkappa(ng,ns)            ! SE shocks to the utility cost of education


    ! parameter structure

    integer, parameter :: nParamMax = 200          ! Maximum number of parameters 
    type param_t
        integer        :: n                        ! Number of parameters loaded from file
        integer        :: nToEstimate              ! Number of parameters to be estimated (depends on T/F in file) DELETE
        character(15)  :: name(nParamMax)          ! Parameter names
        real(wp)       :: val(nParamMax)           ! Parameter values
        real(wp)       :: lb(nParamMax)            ! tight lower bound for parameter values
        real(wp)       :: ub(nParamMax)            ! tight upper bound for parameter values
        logical        :: toEstimate(nParamMax)    ! Which parameters are to be estimated 
        real(wp)       :: se(nParamMax)            ! SE of estimate
        real(wp)       :: bscale(nParamMax)
        real(wp)       :: bmean(nParamMax)
        real(wp)       :: b(nParamMax)             ! rescaled parameter values: for estimation
        real(wp)       :: blb(nParamMax)           ! rescaled parameter values: lower bound
        real(wp)       :: bub(nParamMax)           ! rescaled parameter values: upper bound
    end type param_t
    type(param_t)      :: paramWL, paramS

contains 
    

    !
    ! Copy parameters into relevant arrays and populate remainder of those arrays (WL)
    !
    subroutine paramCopyWL(par)
        use momData, only : Psmsf
        implicit none
        type(param_t),  intent(in)  :: par
        integer :: dbk1, dbk2, s, sm, sf, g, th, ieta, i


        ! other parameters in earnings
        i=12
        do g=1,ng
        do s=1,ns
            i=i+1        
            lny_a(g,s)   = par%val(i)
            lny_asq(g,s) = par%val(i+6)
            lny_acb(g,s) = par%val(i+12)
            Vxi(g,s)     = par%val(i+18)
            Vme(g,s)     = par%val(i+24)
            kappa(g,s)   = par%val(i+30)
        end do
        end do
        Exi = 0.0_wp
        
        ! preferences for working
        i=48
        do g=1,ng
        do s=1,ns
            i=i+1;  
            alphaC0(g,s)    = par%val(i)
            alphaS0(g,s)    = par%val(i+6)
            eta(1,g,s)      = 0.0_wp            !normalisation
            eta(2,g,s)      = par%val(i+12)
            alphaC_a(g,s)   = par%val(i+18)
            alphaC_asq(g,s) = par%val(i+24)
            alphaC_acb(g,s) = par%val(i+30)
            alphaS_a(g,s)   = par%val(i+36)
            alphaS_asq(g,s) = par%val(i+42)
            alphaS_acb(g,s) = par%val(i+48)
            Vu(g,s)         = par%val(i+54)
            peta(g,s)       = par%val(i+60)
        end do
        end do

        ! weights in conditional distr of unobserved heterogeneity in productivity: couples
        i=114
        do sm=1,ns
        do sf=1,ns
            i=i+1;  pthC(1,1,sm,sf) = par%val(i)
            i=i+1;  pthC(1,2,sm,sf) = (1.0_wp-pthC(1,1,sm,sf))*par%val(i)
            i=i+1;  pthC(2,1,sm,sf) = (1.0_wp-pthC(1,1,sm,sf)-pthC(1,2,sm,sf))*par%val(i)
            pthC(2,2,sm,sf) = 1.0_wp - pthC(1,1,sm,sf) - pthC(1,2,sm,sf) - pthC(2,1,sm,sf)
        end do
        end do

        ! weights in conditional distr of unobserved heterogeneity in productivity (th=1): singles
        i=141
        do g=1,ng
        do s=1,ns
            i=i+1;  pthS(g,1,s) = par%val(i)      ! theta=1
            pthS(g,2,s) = 1.0_wp-pthS(g,1,s)      ! theta=2
        end do
        end do
        
        ! construct distribution of theta
        call calcDistrTheta(Psmsf, PthS, PthC, Pth)

        ! Earnings
        i=0
        do g=1,ng
            i=i+1; meanlny_s1(g) = par%val(i)                       ! mean log earnings for s=1, age 23
        end do
        do g=1,ng
        do s=1,ns
            i=i+1; thetaval(g,2,s) = par%val(i)                     ! unobserved heterogeneity in earnings: high ability
            thetaVal(g,1,s) = - pth(g,2)*thetaval(g,2,s)/pth(g,1)   ! low ability
        end do        
        end do     
        returnS = 0.0_wp   
        do g=1,ng
        do s=2,ns
            i=i+1; returnS(g,s) = par%val(i)                        ! value of education in earnings
        end do
        end do

        do g=1,ng
        do th=1,ntheta
        do s=1,ns
            lny(g,th,s) = meanlny_s1(g) + returnS(g,s) + thetaval(g,th,s)
        end do
        end do
        end do

    end subroutine paramCopyWL
    

    !
    ! Calculate parameters derived from those being estimated (WL)
    !
    subroutine derived_parWL
        implicit none
        integer :: a, sm, sf
        
        ! Wage rates by gender and education
        lnyC = lny + log(1._wp-taxC)          ! net earnings rate for couples in logs
        lnyS = lny + log(1._wp-taxS)          ! net earnings rate for single in logs

        ! st errors of shocks
        SExi = sqrt(Vxi)                      ! se of innov in productivity
        SEe  = sqrt(Vxi / (1.0_wp-kappa**2))  ! se of productivity shocks
        SEme = sqrt(Vme)                      ! se of measurement error in earnings
        SEu  = sqrt(Vu)                       ! se of preference shocks
        
        ! utility parameters
        iBR  = 1.0_wp/(beta*R)
        i2pQ = 1.0_wp/(2.0_wp*pQ)
        i4pQ = 1.0_wp/(4.0_wp*pQ)
        lnpQ = log(pQ)

        ! min private and public consumption
        QminS = transfS*0.2                   ! lower bound on Q for singles
        CminS = transfS*0.2 
        QminC = QminS*1.5_wp                  ! eq scale
        CminC = QminS*1.5_wp

    end subroutine derived_parWL
    

    !
    ! Calculate marginal distribution of theta among men and women: Pth(gender,theta)
    !
    subroutine calcDistrTheta(Msmsf, pdf_thS, pdf_thC, pdf_th)
        implicit none
        real(wp), intent(in)  :: Msmsf(0:ns,0:ns), pdf_thS(ng,ntheta,ns), pdf_thC(ntheta,ntheta,ns,ns)
        real(wp), intent(out) :: pdf_th(ng,ntheta)
        integer :: s, ss, ith, iths, ihs, g

        pdf_th = 0.0_wp 
        do ith=1,ntheta                 
            pdf_th(1,ith) = sum(pdf_thS(1,ith,1:ns)*Msmsf(1:ns,0))                                ! single men, th
            pdf_th(2,ith) = sum(pdf_thS(2,ith,1:ns)*Msmsf(0,1:ns))                                ! single women, th
            do iths=1,ntheta
                pdf_th(1,ith) = pdf_th(1,ith) + sum(pdf_thC(ith,iths,1:ns,1:ns)*Msmsf(1:ns,1:ns)) ! married men
                pdf_th(2,ith) = pdf_th(2,ith) + sum(pdf_thC(iths,ith,1:ns,1:ns)*Msmsf(1:ns,1:ns)) ! married women
            end do
            pdf_th(1,ith) = pdf_th(1,ith)/sum(Msmsf(1:ns,:))
            pdf_th(2,ith) = pdf_th(2,ith)/sum(Msmsf(:,1:ns))
        end do
        
    end subroutine calcDistrTheta


    !
    ! copy parameters into their locations MOVE TO DATA??
    !
    subroutine paramCopyS(par, npar, econgain)
        implicit none
        integer, intent(in) :: npar
        real(wp),intent(in) :: par(npar), econgain(0:nH,0:nH)
        integer             :: dbk1, dbk2, s, sm, sf, g, th, ieta, i, mh, wh, ih, ph, ps, d


        i=0

        ! cost of education (=0 for secondary level)
        costS = 0.0_wp
        do g=1,ng
        do s=2,ns
            i=i+1;  costS(g,s,:,0) = par(i)   ! constant 
            i=i+1;  costS(g,s,:,1) = par(i)   ! parental Y: residual
        end do
        end do

        ! expected value of HC for those not marrying (from LC problem)
        mxU = 0.0_wp
        mxU(1,1:nH,0) = econgain(1:nH,0)  ! men
        mxU(2,0,1:nH) = econgain(0,1:nH)  ! women

        ! expected value of HC for married men/women
        do mh=1,nH
        do wh=1,nH
            i=i+1
            shareM(mh,wh) = par(i)
            mxU(1,mh,wh)  = par(i)*econgain(mh,wh)         ! men
            mxU(2,mh,wh)  = econgain(mh,wh)-mxU(1,mh,wh)   ! women
        end do
        end do

        ! additive contant: pref for remaining single
        prefC0(1,:,:) = 0.0_wp
        do g=1,ng
        do th=1,ntheta
        do s=1,ns
            i=i+1;  prefC0(g,th,s) = par(i)
        end do
        end do
        end do
        
        ! additive contant: pref for marrying ps
        prefC1 = 0.0_wp
        do g=1,ng
        do th=1,ntheta
        do d=1,2
            i=i+1;  prefC1(g,th,d) = par(i)
        end do
        end do
        end do
        
        scaleU(:) = 0.01_wp

    end subroutine paramCopyS


end module parameters



! --------------------------------------------------------------------------------------------------------------------
! Grids to discretise state space
!
module grids

    use precision,  only : wp
    use constants,  only : ng, ns, ntheta, mina, maxa

    ! number of poins in grids and matrices
    integer,parameter :: ngpu=8                              ! pref for leisure (u)
    integer,parameter :: ngpe=12                             ! productivity shock (e)
    integer,parameter :: ngpk=16                             ! grid in savings (k)

    ! grids & transition matrices
    real(wp) :: gu(ng,ns,ngpu)                               ! grid in pref for leisure (u)
    real(wp) :: ge(ng,ns,ngpe)                               ! grid in productivity
    real(wp) :: gxi(ng,ns,ngpe)                              ! grid in innovation to productivity
    real(wp) :: TMe(ng,ns,mina:maxa,ngpe,ngpe)               ! transition matrix in producticvity, dimensions are: (g,s,a,ie_1,ie)
    real(wp) :: gKC(ngpk,ns,ns,ntheta,ntheta,mina-1:maxa)    ! grid in assets for couples, by (sm,sf,a,k) 
    real(wp) :: gKS(ngpk,ns,ntheta,mina-1:maxa,ng)           ! grid in assets for singles, by (g,s,a,k) 

end module grids



! --------------------------------------------------------------------------------------------------------------------
! simulation  structure
!
module SimStructure

    use precision, only : wp
    use constants, only : mina, maxa, ng, ns, nH
    implicit none

    ! for individuals in couples and singles: structures for simulations of the entire LC
    integer, parameter :: nrepsS=10, nrepsWL=10
        
    type structWL 
        real(wp) :: k(mina:maxa)         ! assets
        real(wp) :: e(ng,mina:maxa)      ! productivity
        real(wp) :: w(ng,mina:maxa)      ! annual earnings
        real(wp) :: lWres(ng,mina:maxa)  ! residuals log earnings
        real(wp) :: u(ng,mina:maxa)      ! preference for leisure: women and men
        real(wp) :: inc(mina:maxa)       ! family net income
        real(wp) :: V(mina:maxa)         ! family EV of present and future U, conditional on today's optimal choice (Q,C,L)
        integer  :: L(ng,mina:maxa)      ! male and female leisure 
        real(wp) :: alpha(ng,mina:maxa)  ! preferences for leisure
        real(wp) :: Q(mina:maxa)         ! public consumption
        integer  :: ieta(ng)             ! permanent pref for leisure
        integer  :: itheta(ng)           ! ability
        real(wp) :: s(ng)                ! education
        real(wp) :: discY(ng)            ! total lifetime discounted labour income
        real(wp) :: privC(ng,mina:maxa)  ! private consumption
        real(wp) :: ut(ng,mina:maxa)     ! utility per period
        real(wp) :: insample(mina:maxa)        
    end type structWL

    type structSpH
        integer  :: s                            ! choice: simulated education 
        integer  :: h                            ! choice: simulated human capital 
        integer  :: ph                           ! choice: simulated partner's h
        real(wp) :: cS(ns)                       ! total cost of education
        real(wp) :: kappa(ns)                    ! unobserved cost of eduction
        real(wp) :: probSpH_byK(ns,0:nH)         ! probability of (S,pH) conditional on (theta, X, kappa)
        real(wp) :: eps(0:nH)                    ! unobserved preferences for partners
        real(wp) :: VSpH(ns,0:nH)
        integer  :: phbyS(ns)
        integer  :: h0                           ! choice: simulated human capital 
        integer  :: ph0                          ! choice: simulated partner's h
    end type structSpH

    type structLC
        integer  :: coupleDta                    ! dummy for couple during WL
        integer  :: insampleWL                   ! observation in BHPS data for WL period
        integer  :: insampleS                    ! observation in BHPS data for education/marriage period
        integer  :: s(ng)                        ! education
        integer  :: female                       ! dummy for woman
        real(wp) :: pYres                        ! residual parental income
        integer  :: age1                         ! age 1st observed (only relevant for WL)
        integer  :: ageN                         ! age last observed (only relevant for WL) 
        real(wp) :: EcS(ns)                      ! expected total cost of education
        real(wp) :: rvtheta                      ! ability (random draw from uniform)
        integer  :: theta                        ! ability
        real(wp) :: probSpH(ns,0:nH)             ! probability of (S,pH) conditional on (theta, X)
        real(wp) :: probS(ns)                    ! probability of (S) conditional on (theta, X)
        real(wp) :: probSpS(ns,0:ns)             ! probability of (S,pS) conditional on (theta, X)
        real(wp) :: V0(ns)                       ! value of each education level at conditional (on educ) optimal marital choice
        real(wp) :: V1(ns)                       ! value of each education level at unconditional optimal marital choice
        type(structSpH) :: SpH(nrepsS)           ! education and sorting variables
        type(structWL)  :: wl(nrepsS,nrepsWL)    ! WL variables
    end type structLC

    type(structLC), allocatable, target :: simLC(:)
    integer :: nobsLC, nobsSregr                   ! nr observations in data: to be read and shared across routines
    
end module SimStructure


