! --------------------------------------------------------------------------------------------------------------------
! This file is part of the package EMar, developed to produce the results in the paper "The marriage market, labor 
!    supply and education choice," by P.A. Chiapppori, M. Costa-Dias and C. Meghir. 
! Copyright (c) 2017 Monica Costa Dias
!
! Routines to simulate sensitivity to risk and policy counterfactuals
! --------------------------------------------------------------------------------------------------------------------

module experiments

use paths
use precision,      only : wp
use constants,      only : ng, ns, nH
use parameters,     only : nParamMax, paramS, paramWL, shareM, Exi, Vxi, paramcopyWL
use IOroutines,     only : loadDataMomSpH, writeSharing, writePth, writeGain, writeSurplus, writeSorting
use SimStructure,   only : simLC, nobsLC
use solutionWL,     only : SolveWL
use solutionSpH,    only : calcGains, SolveAndSimSpH, sortingmx, calcSorting, calcQformSpH, &
                           momSpH, calcDist_probSpH, InitCondSpH
use estimationSpH,  only : objcalcSpH
use VstartLife,     only : surplus

integer,  parameter :: npts=9
real(wp), private   :: sortingbyrisk(0:npts,0:nH,0:nH), surplusbyrisk(0:npts,nH,nH), aggSurplus(0:npts)

contains



    ! ----------------------------------------------------------------------------------------------------------------
    ! Simulate policy reforms: education subsidy
    !

    subroutine EducSubsidy
        implicit none
        real(wp), parameter :: deltaC=0.1_wp
        real(wp)            :: Qform, b(nParamMax), Ecost(ns)
        integer             :: checkMkt, mktclear, dS, dpH, violations, unit, s 


        ! open output file to export results
        unit = 4354
        open (unit, file=resultspath//'EducSubsidy.txt', action='write')
        

        ! solve problem at baseline
        checkMkt = 1
        call SolveAndSimSpH(.false., ' ',.false., ' ', checkMkt, Qform)

        ! calculate average cost of education
        forall(s=1:ns) Ecost(s) = sum(simLC(:)%EcS(s))/real(nobsLC,wp)
        
        ! print results to file
        write(unit, '(/, "------------------------------------------------------------------------", &
                      /, "Baseline", /, "------------------------------------------------------------------------", /)')
        write(unit, '("Expected cost of education (s=1,2,3): ", 3F12.6, /)') Ecost(1), Ecost(2), Ecost(3)
        call writePth(unit)
        call writeGain(unit)
        call writeSurplus(unit)
        call writeSorting(unit, sortingmx)
        call writeSharing(unit)
        
        ! store parameters
        b = paramS%val

        ! solve problem with university subsidy
        paramS%val(3) = paramS%val(3) - deltaC*Ecost(3)            
        paramS%val(7) = paramS%val(7) - deltaC*Ecost(3)
        checkMkt  = 0
        mktclear  = 1
        dS        = 1
        dpH       = 1
        call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)

        violations = sum( merge( 1, 0, shareM(:,:)<=0.0101_wp .or. shareM(:,:)>=0.9899_wp ) )
        if (violations>0) then
            checkMkt = 1
            call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)
        end if
        
        ! print results to file
        write(unit, '(/,/, "------------------------------------------------------------------------", &
                      /, "Low cost University", /, "------------------------------------------------------------------------")')
        call writeSorting(unit, sortingmx)
        call writeSharing(unit)
        close (unit)
        
        ! recover parameters
        paramS%val = b        
        
    end subroutine EducSubsidy



    ! ----------------------------------------------------------------------------------------------------------------
    ! Sensitivity to risk in productivity
    !

    subroutine sensitivityrisk
        implicit none
        integer  :: checkMkt, mktclear, dS, dpH, p
        real(wp) :: prisk, Exi0(ng,ns), Qform, bSpH(nParamMax), bWL(nParamMax)


        ! Initialse matrices to zero
        sortingbyrisk = 0.0_wp
        surplusbyrisk = 0.0_wp

        ! Set solution conditions
        checkMkt  = 1
        mktclear  = 1
        dS        = 1
        dpH       = 1

        ! Solve problem at baseline level of risk, then store surplus and marital sorting patterns
        p = 0                                                                      ! index for baseline
        call SolveAndSimSpH(.false., ' ', .false., ' ', 1, Qform)         ! solve marriage and education problem
        call storeRiskSim(p)                                                       ! store surplus and marital sorting
        
        ! Store baseline values of parameters
        Exi0 = Exi
        bSpH = paramS%val
        bWL  = paramWL%val
        
        ! Change risk, recalculate market equilibrium and measure surplus        
        do p=1,npts
        
            prisk = 0.1_wp * real(4+2*(p-1),wp)                    ! set new level of earnings risk
            paramWL%val(31:36) = paramWL%val(31:36) * prisk        ! update variance of earnings
            call paramcopyWL(paramWL)                              ! load new parameters
            Exi = Exi0 + 0.5_wp * (1.0_wp-prisk) * Vxi             ! and update expected value of shock (log normal)            
            call SolveWL                                           ! solve WL problem
            call calcGains                                         ! calculate gains from marriage
            call calcDist_probSpH                                  ! calculate distribution of marital patterns: p(S,pH | g,th)
            call loadDataMomSpH(momSpH)                            ! load data moments for SpH
            call InitCondSpH                                       ! read initial conditions & shocks
            call objcalcSpH (mktclear, checkMkt, dS, dpH, Qform)   ! solve for new sharing rule that clears market
            call storeRiskSim(p)                                   ! store surplus and marital sorting
            write(*,'(/,"      equilibrium sharing rule calculated for p=", I2, /)'), p

            paramS%val  = bSpH                                     ! recover original paramters
            paramWL%val = bWL

        end do
                
        ! export results
        call printRiskResults(npts, sortingbyrisk, aggSurplus, resultsPath//'risk results.txt')
        
    end subroutine sensitivityrisk


    !
    ! Store results
    !
    subroutine storeRiskSim(p)
        implicit none
        integer, intent(in) :: p            

        ! recover sorting patterns and marital surplus
        call calcSorting
        call calcGains
        sortingbyrisk(p,:,0)       = sortingmx(1,:,0)
        sortingbyrisk(p,0,:)       = sortingmx(2,0,:)
        sortingbyrisk(p,1:nH,1:nH) = (sortingmx(1,1:nH,1:nH)+sortingmx(2,1:nH,1:nH)) / 2.0_wp
        surplusbyrisk(p,:,:)       = surplus

        ! calculate aggregate surplus with baseline sorting and new sorting weights
        aggSurplus(p) = sum( surplusbyrisk(p,:,:) * sortingbyrisk(0,1:nH,1:nH) ) / sum(sortingbyrisk(0,1:nH,1:nH))

    end subroutine storeRiskSim


    !
    ! print risk results to file
    !

    subroutine printRiskResults(nrisk, sortingbyrisk, aggSurplus, filename)
        implicit none
        integer, intent(in)      :: nrisk
        real(wp), intent(in)     :: sortingbyrisk(0:nrisk,0:nH,0:nH), aggSurplus(0:nrisk)
        character(*), intent(in) :: filename
        integer :: p, mh, wh, resultsUnit, s
        
        
        resultsUnit = 768
        open (resultsUnit, file=filename, action='write')

        write(resultsUnit, '(/, "---------------------------------------------------------------------------")')
        write(resultsUnit, '("sorting patterns Hm / Hf")')
        write(resultsUnit,'("   ", 7A9)') "wh0", "wh1", "wh2", "wh3", "wh4", "wh5", "wh6"
        do p=0,npts        
            write(resultsUnit, '(/,"risk level", I2)') p
            do mh=0,nH
                write(resultsUnit,'("mh", I1, 7F9.2)') mh, (sortingbyrisk(p,mh,wh), wh=0,nH)
            end do
        end do
        
        write(resultsUnit, '(/, "---------------------------------------------------------------------------")')
        write(resultsUnit, '("Aggregate surplus and effect against baseline")')
        write(resultsUnit, '(/,"p  S:baselineW    rel diff")')
        do p=1,npts        
            write(resultsUnit, '(I2, 2F12.3)') p, aggSurplus(p), (aggSurplus(p)-aggSurplus(0))/aggSurplus(0)
        end do

        close (resultsUnit)
        
    end subroutine printRiskResults

end module experiments

