! -------------------------------------------------------------------------------------------------------
! Auxilliary functions
! -------------------------------------------------------------------------------------------------------


module functions

use precision,  only : wp
use constants,  only : mina, maxa, enda, ns, ntheta, ng
use parameters, only : lnY_a, lnY_asq, lnY_acb, R, eta, costS, pQ, SEu, SExi, Exi, SEe, kappa, &
                       lnYC, alphaC0, alphaC_a, alphaC_asq, alphaC_acb, transfC, CminC, QminC, &
                       lnYS, alphaS0, alphaS_a, alphaS_asq, alphaS_acb, transfS, CminS, QminS
use grids,      only : gE, ngpe, gKS, gKC, ngpk, gU, ngpu, gxi, TMe

real(wp), private :: minKC(mina-1:enda), minKS(mina-1:enda), &       ! couples and singles: lower bounds in assets depend on transfers only
                     maxKC(ns,ns,ntheta,ntheta,mina-1:maxa), &       ! couples: upper bound in assets by (sm,sf,thetam,thetaf,a)
                     maxKS(ns,ntheta,mina-1:maxa,ng)                 ! singles: upper bound in assets by (s,theta,a,g)

contains 


    ! -------------------------------------------------------------------------------------------------
    ! income functions
    !    

    real(wp) function earnings(g,s,lnyrate,a,e)
        implicit none
        integer,  intent(in) :: g,s,a
        real(wp), intent(in) :: lnyrate,e
        real(wp)             :: age,agesq,agecb
        
        age      = 0.1_wp*real(min(a,50)-23,wp)     !assume no earnings movements from a=50 onwards
        agesq    = age*age
        agecb    = agesq*age
        earnings = exp(lnyrate + lny_a(g,s)*age + lny_asq(g,s)*agesq  + lny_acb(g,s)*agecb + e)
       
    end function earnings
    

    pure real(wp) function logWres(w,g,s,a)
        implicit none
        integer,  intent(in) :: g,s,a
        real(wp), intent(in) :: w
        real(wp)             :: age,agesq,agecb
        
        age      = 0.1_wp*real(min(a,50)-23,wp)    !assume no earnings movements from a=50 onwards
        agesq    = age*age
        agecb    = agesq*age
        logWres  = log(w) - lny_a(g,s)*age - lny_asq(g,s)*agesq - lny_acb(g,s)*agecb
       
    end function logWres
    

    pure real(wp) function resourcesC(ym,yf,Lm,Lf,k_1,k)
        implicit none
        integer,  intent(in) :: Lm, Lf
        real(wp), intent(in) :: ym,yf,k_1,k
        
        resourcesC = transfC*real(Lm+Lf,wp) + ym*real(1-Lm,wp) + yf*real(1-Lf,wp) + R*k_1 - k
       
    end function resourcesC


    pure real(wp) function resourcesS(w,L,k_1,k)
        implicit none
        integer,  intent(in) :: L
        real(wp), intent(in) :: w,k_1,k
        
        resourcesS = transfS*real(L,wp) + w*real(1-L,wp) + R*k_1 - k
       
    end function resourcesS




    ! -------------------------------------------------------------------------------------------------
    ! utility factors
    !
    
    pure real(wp) function utC(g,ieta,s,a,u)
        implicit none
        integer,  intent(in) :: g,ieta,s,a
        real(wp), intent(in) :: u
        real(wp)             :: age,agesq,agecb
        
        age   = 0.1_wp*real(a-23,wp)
        agesq = age*age
        agecb = age*age*age
        utC  = exp(alphaC0(g,s) + alphaC_a(g,s)*age + alphaC_asq(g,s)*agesq + alphaC_acb(g,s)*agecb + eta(ieta,g,s) + u) 
        
    end function utC
    
    
    pure real(wp) function utS(g,ieta,s,a,u)
        implicit none
        integer,  intent(in) :: g,ieta,s,a
        real(wp), intent(in) :: u
        real(wp)             :: age,agesq,agecb
        
        age   = 0.1_wp*real(a-23,wp)
        agesq = age*age
        agecb = age*age*age
        utS  = exp(alphaS0(g,s) + alphaS_a(g,s)*age + alphaS_asq(g,s)*agesq + alphaS_acb(g,s)*agecb + eta(ieta,g,s) + u) 
        
    end function utS




    ! -------------------------------------------------------------------------------------------------
    ! cost of education
    !    
    
    pure real(wp) function costEduc (g, th, s, pYres, kappa)
        implicit none
        integer, intent(in)  :: g, th, s
        real(wp), intent(in) :: pYres, kappa
        
        costEduc = costS(g,s,th,0) + costS(g,s,th,1)*pYres + kappa
        
    end function costEduc
        



    ! -------------------------------------------------------------------------------------------------
    ! human capital
    ! 
       
    integer function getH(ith,s)
        integer,intent(in) :: ith,s
        
        getH = (ith-1)*ns + s
        
    end function getH
    
    integer function recoverS(ih)
        integer,intent(in) :: ih
        
        recoverS = merge(ih,ih-3,ih<3.5_wp)
        
    end function recoverS
    
    integer function recoverTh(ih)
        integer,intent(in) :: ih
        
        recoverTh = merge(1,2,ih<3.5_wp)
        
    end function recoverTh



    ! -------------------------------------------------------------------------------------------------
    ! Grid functions
    !
    
    !
    ! Grid in savings (K) for married couples and singles
    ! Savings at age a are those transferred to age a+1 so bounds define the set of possible choices 
    !     at age a. Start with k(mina-1)=0 meaning that everyone starts life with 0 savings.
    !
    subroutine getGridK
        implicit none
        real(wp),parameter :: ratio=1.25_wp              ! change ration to alter distribution of grid points
        integer            :: a, s, sm, sf, g, ki, itm, itf, it
        real(wp)           :: maxYm, maxYf, maxY, unit


        ! compute bounds in savings decisions at time a
        maxKC = 0.0_wp      ! upper bound for savings among married couples
        maxKS = 0.0_wp      ! upper bound for savings among singles
        
        ! for married couples
        do itm = 1,ntheta
        do itf = 1,ntheta
        do sm  = 1,ns
        do sf  = 1,ns
        do a = mina, maxa     
            maxYm                  = earnings( 1, sm, lnyC(1,itm,sm), a, ge(1,sm,ngpe) )
            maxYf                  = earnings( 2, sf, lnyC(2,itf,sf), a, ge(2,sf,ngpe) )
            maxKC(sm,sf,itm,itf,a) = maxKC(sm,sf,itm,itf,a-1)*R + maxYm + maxYf - CminC - pQ*QminC
        end do
        end do
        end do
        end do
        end do

        minKC = 0.0_wp
        do a = maxa-1, mina, -1
            minKC(a) = (R**(-1)) * ( minKC(a+1) + CminC + pQ*QminC - 2.0_wp*transfC )
        end do
            
        ! for singles
        do g  = 1,ng
            do it = 1,ntheta
            do s  = 1,ns
            do a  = mina, maxa     
                maxY            = earnings(g, s, lnyS(g,it,s), a, ge(g,s,ngpe) )
                maxKS(s,it,a,g) = maxKS(s,it,a-1,g)*R + maxY - CminS - pQ*QminS
            end do
            end do
            end do
        end do
        
        minKS = 0.0_wp
        do a = maxa-1, mina, -1
            minKS(a) = (R**(-1)) * ( minKS(a+1) + CminS + pQ*QminS - transfS )
        end do
        
        
        ! calculate grid
        gKS = 0.0_wp
        gKC = 0.0_wp
        forall(a=mina-1:maxa)
            gKS(1,:,:,a,:)   = minKS(a)
            gKC(1,:,:,:,:,a) = minKC(a)
        end forall
        
        do a=mina,maxa

            ! for married couples
            do itm = 1,ntheta
            do itf = 1,ntheta
            do sm  = 1,ns
            do sf  = 1,ns
                unit = (maxKC(sm,sf,itm,itf,a)-minKC(a)) * (1.0_wp-ratio) / (1.0_wp-ratio**(ngpk-1))
                do ki=2,ngpk
                    gKC(ki,sm,sf,itm,itf,a) = gKC(ki-1,sm,sf,itm,itf,a) + unit*ratio**(ki-2)
                end do !ki
            end do
            end do
            end do
            end do

            ! for singles
            do g  = 1,ng
            do it = 1,ntheta
            do s  = 1,ns
                unit = (maxKS(s,it,a,g)-minKS(a)) * (1.0_wp-ratio) / (1.0_wp-ratio**(ngpk-1))
                do ki=2,ngpk
                    gKS(ki,s,it,a,g) = gKS(ki-1,s,it,a,g) + unit*ratio**(ki-2)
                end do
            end do
            end do
            end do
            
       end do
       
    end subroutine getGridK



    ! 
    ! Construct grids in shocks: HC (or ability) and productivity
    !
    subroutine getGridShocks
        implicit none
        integer :: g, s, a, ie_1, L, iu

        do g = 1,ng
        do s = 1,ns

            ! preferences for leisure
            call getgrid_N ( ngpu, 0.0_wp, SEu(g,s), gu(g,s,:),  1, 98.0_wp )    ! grid in u (in logs) - central 98% of distribution

            ! earnings shocks
            call getgrid_N ( ngpe, Exi(g,s), SExi(g,s), gxi(g,s,:), 0, 98.0_wp ) ! grid in xi (productivity innovation, in log(earnings) units)
            call getgrid_N ( ngpe, 0.0_wp, SEe(g,s), ge(g,s,:), 0, 98.0_wp  )    ! grid in productivity shock e in log(earnings) units (does not depend on age - AR(1) in equilibrium)
            
            ! transitions in productivity: do not depend on age but keep dependence in case want to relax this later
            do a=mina+1,maxa
            do ie_1=1,ngpe                                                   ! poins in grid last period (a-1)
                call trprob(ge(g,s,ie_1), kappa(g,s), 0.0_wp, SExi(g,s), ngpe, ge(g,s,:), TMe(g,s,a,ie_1,:))
            end do
            end do
            TMe(g,s,mina,:,:) = 1.0_wp / real(ngpe,wp)

        end do
        end do
        
    end subroutine getGridShocks



    !
    ! Calculate transition matrix for an AR(1) process
    !
    subroutine trprob(rv,ar,Einnov,sinnov,ngp,grd1,trp)
        use precision, only : wp
        use nag_g_ib,  only : g01eaf
        implicit none
        real(wp), intent(in)  :: rv          ! value of random variable in current period
        real(wp), intent(in)  :: ar          ! AR(1) persistence parameter
        real(wp), intent(in)  :: Einnov      ! Expected value of innovation
        real(wp), intent(in)  :: sinnov      ! Standard error of innovation
        integer,  intent(in)  :: ngp         ! Number of grid poins
        real(wp), intent(in)  :: grd1(ngp)   ! grid rv next period
        real(wp), intent(out) :: trp(ngp)    ! Output vector of transition probabilities
        real(wp) mean1                       ! mean of rv next period
        integer  i, ifail

        
        ! compute mean of rv1 (random variable next period); se is sinnov
        mean1 = ar*rv + Einnov
        
        ! probability of transition to intervals where each grid point is the middle probability point
        ifail = 0
        trp(1) = (g01eaf('L',(grd1(2)-mean1)/sinnov,ifail) + g01eaf('L',(grd1(1)-mean1)/sinnov,ifail))/2.0_wp
        do i=2,ngp-1
            trp(i) = (g01eaf('L',(grd1(i+1)-mean1)/sinnov,ifail) - g01eaf('L',(grd1(i-1)-mean1)/sinnov,ifail))/2.0_wp
        end do
        trp(ngp) = 1.0_wp - (g01eaf('L',(grd1(ngp-1)-mean1)/sinnov,ifail) + g01eaf('L',(grd1(ngp)-mean1)/sinnov,ifail))/2.0_wp

    end subroutine trprob



    !
    ! Get equally spaced (in probabilities) grid points for a normal(mean,se)
    !
    subroutine getgrid_N(ngp,mean,se,grid,dbounds,region)
        implicit none
        integer,  intent(in)  :: ngp, dbounds
        real(wp), intent(in)  :: mean, se, region
        real(wp), intent(out) :: grid(ngp)
        integer i, ifail
        real(wp) step, lb, x
        real(wp),external :: g01faf
    
    
        lb   = 0.01_wp*(0.5_wp*(100.0_wp-region))

        if (dbounds==1) then                                            ! if grid includes the boundaries of the interval

            step = 0.01_wp*region/real(ngp-1,wp)
            do i=1,ngp
                ifail   = -1
                x       = g01faf('L', lb+step*real(i-1,wp), ifail)
                grid(i) = x*se+mean
            end do

        else                                                            ! if grid includes midpoint in each subinterval

            step = 0.01_wp*region/real(ngp,wp)
            do i=1,ngp
                ifail   = -1
                x       = g01faf('L', lb+step*(real(i,wp)-.5_wp), ifail)
                grid(i) = x*se+mean
            end do
            
        end if

    end subroutine getgrid_N
    



    ! -------------------------------------------------------------------------------------------------
    ! find the bounds for x in vector of size ngp where vector is order in an 
    !    increasing (if dincreasing=1) or decreasing (else) way; outputs the ub index iub
    !    
    
    subroutine getPosition(x,ngp,vector,iub,dincreasing)
        implicit none
        integer,intent(in)  :: ngp,dincreasing
        real(8),intent(in)  :: x,vector(ngp)
        integer,intent(out) :: iub
        integer i
        
        if (dincreasing==1) then
            do i=2,ngp-1
                if (x<=vector(i)) exit
            end do
        else
            do i=2,ngp-1
                if (x>=vector(i)) exit
            end do
        end if
        iub = i
            
    end subroutine getPosition



    ! -------------------------------------------------------------------------------------------------
    ! Invert square matrix
    !    
    
    subroutine invMx (n,A,iA)
        use nag_f_ib,   only : f07maf
        implicit none
        integer, intent(in)   :: n
        real(wp), intent(in)  :: A(n,n)
        real(wp), intent(out) :: iA(n,n)
        real(wp)              :: Aaux(n,n), wkspace(n)
        integer               :: i, ifail, permute(n)
        
        iA = 0.0_wp
        forall(i=1:n) iA(i,i) = 1.0_wp
        Aaux = A
        call f07maf ( 'U', n, n, Aaux, n, permute, iA, n, wkspace, n, ifail)
        
    end subroutine invMx



    ! -------------------------------------------------------------------------------------------------
    ! Statistical functions
    !
    
    ! density of standardised normal
    real(8) function densitySN(x)
        implicit none
        real(8),intent(IN) :: x
        real(8) y
        real(8),external :: X01AAF
        
        densitySN = exp(-x**2/2)/sqrt(2*X01AAF(y))

    end function densitySN
    
    
    ! cumulative distribution of logistic
    real(8) function cdf_logistic(x,se)
        implicit none
        real(8),intent(in) :: x, se
        
        cdf_logistic = 1.d0 / ( 1.d0 + exp( - x / se ) )
    
    end function cdf_logistic


    ! truncated density of 2d normal distribution with zero mean
    real(8) function truncatedpdf_N2d(x,y,sx,sy,tale)
        implicit none
        real(8),intent(in) :: x,y,sx,sy,tale

        truncatedpdf_N2d = densitySN(x/sx) * densitySN(y/sy) / (sx*sy*((1.d0-2.d0*tale)**2))

    end function truncatedpdf_N2d


    ! truncated density of 1d normal distribution with zero mean
    real(8) function truncatedpdf_N1d(x,sx,tale)
        implicit none
        real(8),intent(in) :: x,sx,tale

        truncatedpdf_N1d = densitySN(x/sx) / (sx*(1.d0-2.d0*tale))

    end function truncatedpdf_N1d




    ! -------------------------------------------------------------------------------------------------
    ! Linear interpolation functions
    !
    
    ! 1D linear interpolation
	elemental real(8) function LinInterp1D (x,x1,x2,f1,f2)
		implicit none
		real(8),intent(in) :: x, x1, x2, f1, f2
		real(8)            :: dx

		if (x1==x2 .OR. f1==f2) then
			LinInterp1d = f1
		else
			dx = (x-x1)/(x2-x1)
			LinInterp1d = f1+dx*(f2-f1)
		end if

	end function LinInterp1d
	
	
    ! Simplical 2D linear interpolation
    elemental real(8) function LinInterp2D (x,x1,x2,y,y1,y2,f11,f12,f21,f22)
		implicit none
		real(8), intent(in) :: x,y,x1,x2,y1,y2,f11,f12,f21,f22
		real(8)             :: f1,f2

		if (x1==x2) then
			LinInterp2d = LinInterp1d(y,y1,y2,f11,f22)

		else if (y1==y2) then
			LinInterp2d = LinInterp1d(x,x1,x2,f11,f22)

		else
			f1 = LinInterp1d(y,y1,y2,f11,f12)			! interpolate in y for the 2 pts in x
			f2 = LinInterp1d(y,y1,y2,f21,f22)
			LinInterp2d = LinInterp1d(x,x1,x2,f1,f2)	! interpolate in x
		end if

	end function LinInterp2D
    
    
    
    
    ! -------------------------------------------------------------------------------------------------
    ! Calculate power-of-10 scaling factor so that val is in a range determined by offset
    !
    
    subroutine scaleToRange(val,scale,tval)
        implicit none
        real(wp), intent(in)  :: val
        real(wp), intent(out) :: scale,tval

        scale = floor( log10( max( abs(val), 0.000001_wp ) ) )
        tval  = val / (10.d0**scale)

    end subroutine scaleToRange




    ! -------------------------------------------------------------------------------------------------
    ! converts real to variable length string by calling intToStrLen to
    ! determine return length
    !
    
    pure function realToStr(N)
        implicit none
        real(wp), intent(in)       :: N
        character(realToStrLen(N)) :: realToStr
        character(40)              :: tempStr40
        
        write (tempStr40,*) N
        realToStr = adjustl(tempStr40)

    end function realToStr


    ! returns length of real N as a string    
    pure function realToStrLen(N)
        implicit none
        real(wp), intent(in) :: N
        integer              :: realToStrLen
        character(40)        :: tempStr40
        
        write (tempStr40,*) N
        realToStrLen = len_trim(adjustl(tempStr40))
        
    end function realToStrLen

    
end module functions



